@@ -1,7 +1,7 @@
 /*
  * This file is part of BOINC.
- * http://boinc.berkeley.edu
- * Copyright (C) 2021 University of California
+ * https://boinc.berkeley.edu
+ * Copyright (C) 2025 University of California
  *
  * BOINC is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License
@@ -19,8 +19,14 @@
 package edu.berkeley.boinc
 
 import android.app.ActivityManager.TaskDescription
-import android.content.*
-import android.os.Build
+import android.content.BroadcastReceiver
+import android.content.ComponentName
+import android.content.Context
+import android.content.Intent
+import android.content.IntentFilter
+import android.content.ServiceConnection
+import android.os.Build.VERSION
+import android.os.Build.VERSION_CODES
 import android.os.Bundle
 import android.os.IBinder
 import androidx.appcompat.app.AppCompatActivity
@@ -48,7 +54,6 @@ import edu.berkeley.boinc.utils.getBitmapFromVectorDrawable
 class SplashActivity : AppCompatActivity() {
     private var binding: ActivitySplashBinding? = null
     private var mIsBound = false
-    private var mIsWelcomeSpecificFirstRun = true
     private val mConnection: ServiceConnection = object : ServiceConnection {
         override fun onServiceConnected(className: ComponentName, service: IBinder) {
             // This is called when the connection with the service has been established
@@ -59,8 +64,6 @@ class SplashActivity : AppCompatActivity() {
                 if (!isMutexAcquiredAsync().await()) {
                     showNotExclusiveDialog()
                 }
-                mIsWelcomeSpecificFirstRun =
-                    BuildConfig.BUILD_TYPE.contains("xiaomi") && !monitor!!.welcomeStateFile
                 // Read log level from monitor preferences and adjust accordingly
                 setLogLevel(monitor!!.logLevel)
                 setLogCategories(monitor!!.logCategories)
@@ -79,10 +82,6 @@ class SplashActivity : AppCompatActivity() {
         override fun onReceive(context: Context, intent: Intent) {
             if (mIsBound) {
                 try {
-                    if (mIsWelcomeSpecificFirstRun) {
-                        startActivity(Intent(this@SplashActivity, LicenseActivity::class.java))
-                        return
-                    }
                     when (monitor!!.setupStatus) {
                         ClientStatus.SETUP_STATUS_AVAILABLE -> {
                             Logging.logDebug(Logging.Category.GUI_ACTIVITY, "SplashActivity SETUP_STATUS_AVAILABLE")
@@ -119,14 +118,17 @@ class SplashActivity : AppCompatActivity() {
         setContentView(binding!!.root)
 
         // Use BOINC logo in Recent Apps Switcher
-        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) { // API 21
+        if (VERSION.SDK_INT >= VERSION_CODES.LOLLIPOP) { // API 21
             val label = title.toString()
-            val taskDescription: TaskDescription = if (Build.VERSION.SDK_INT < Build.VERSION_CODES.P) { // API 28
+            val taskDescription: TaskDescription = if (VERSION.SDK_INT < VERSION_CODES.P) { // API 28
                 val icon = this.getBitmapFromVectorDrawable(R.drawable.ic_boinc)
                 @Suppress("DEPRECATION")
                 TaskDescription(label, icon)
-            } else {
+            } else if (VERSION.SDK_INT < VERSION_CODES.TIRAMISU) { // API 28 < x < API 33
+                @Suppress("DEPRECATION")
                 TaskDescription(label, R.drawable.ic_boinc)
+            } else { // > API 33
+                TaskDescription.Builder().setLabel(label).setIcon(R.drawable.ic_boinc).build()
             }
             setTaskDescription(taskDescription)
         }
@@ -193,13 +195,13 @@ class SplashActivity : AppCompatActivity() {
     }
 
     private fun isMutexAcquiredAsync(callback: ((Boolean) -> Unit)? = null) =
-        TaskRunner(callback, {isMutexAcquired()})
+        TaskRunner(callback) { isMutexAcquired() }
 
     private fun isMutexAcquired(): Boolean {
-        var retryCount = 5;
+        var retryCount = 5
         while(!monitor!!.boincMutexAcquired() && retryCount > 0) {
-            Thread.sleep(1000);
-            --retryCount;
+            Thread.sleep(1000)
+            --retryCount
         }
         return monitor!!.boincMutexAcquired()
     }
