@@ -1,7 +1,7 @@
 /*
  * This file is part of BOINC.
- * http://boinc.berkeley.edu
- * Copyright (C) 2020 University of California
+ * https://boinc.berkeley.edu
+ * Copyright (C) 2025 University of California
  *
  * BOINC is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License
@@ -20,8 +20,6 @@
 /*
  * This tries to be the same as lib/error_numbers.h
  */
-@file:JvmName("BOINCErrors")
-
 package edu.berkeley.boinc.utils
 
 // Function return values.
