@@ -1,7 +1,7 @@
 /*
  * This file is part of BOINC.
- * http://boinc.berkeley.edu
- * Copyright (C) 2021 University of California
+ * https://boinc.berkeley.edu
+ * Copyright (C) 2025 University of California
  *
  * BOINC is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License
@@ -42,16 +42,25 @@ import org.robolectric.annotation.Config
 
 @RunWith(RobolectricTestRunner::class)
 class ClientNotificationTest {
+    private lateinit var context: Context
     private lateinit var clientNotification: ClientNotification
+    private lateinit var appPreferences: AppPreferences
+    private lateinit var deviceStatus: DeviceStatus
+    private lateinit var deviceStatusData: DeviceStatusData
 
     @Before
     fun setUp() {
-        clientNotification = ClientNotification(ApplicationProvider.getApplicationContext())
+        context = ApplicationProvider.getApplicationContext()
+        clientNotification = ClientNotification(context)
+        appPreferences = mockkClass(AppPreferences::class)
+        deviceStatusData = DeviceStatusData()
+
+        deviceStatus = DeviceStatus(context, appPreferences, deviceStatusData)
     }
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_NEVER then expect corresponding Notification title and text`() {
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         val notification = clientNotification.buildNotification(clientStatus, true, null)
@@ -61,7 +70,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_IDLE then expect corresponding Notification title and text`() {
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_IDLE
 
         val notification = clientNotification.buildNotification(clientStatus, true, null)
@@ -71,7 +80,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_SUSPENDED then expect corresponding Notification title and text`() {
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_SUSPENDED
 
         val notification = clientNotification.buildNotification(clientStatus, true, null)
@@ -81,7 +90,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_COMPUTING and activeTasks is null then expect corresponding Notification title and text`() {
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
 
         val notification = clientNotification.buildNotification(clientStatus, true, null)
@@ -91,7 +100,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_COMPUTING and activeTasks is empty then expect corresponding Notification title and text`() {
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
 
         val notification = clientNotification.buildNotification(clientStatus, true, listOf())
@@ -102,7 +111,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_COMPUTING and activeTasks contains records then expect corresponding Notification title and text`() {
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
 
         val activeTasks = listOf(
@@ -123,7 +132,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_COMPUTING and activeTasks contains records with Project equal null or App equal null then expect no exception thrown`() {
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
 
         val activeTasks = listOf(
@@ -144,8 +153,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_NEVER then expect only one Notification Action with menu_run_mode_enable title`() {
-        val context = ApplicationProvider.getApplicationContext<Context>()
-        val clientStatus = ClientStatus(context, null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         val notification = clientNotification.buildNotification(clientStatus, true, null)
@@ -155,8 +163,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_IDLE then expect only one Notification Action with menu_run_mode_disable title`() {
-        val context = ApplicationProvider.getApplicationContext<Context>()
-        val clientStatus = ClientStatus(context, null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_IDLE
 
         val notification = clientNotification.buildNotification(clientStatus, true, null)
@@ -166,8 +173,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_SUSPENDED then expect only one Notification Action with menu_run_mode_disable title`() {
-        val context = ApplicationProvider.getApplicationContext<Context>()
-        val clientStatus = ClientStatus(context, null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_SUSPENDED
 
         val notification = clientNotification.buildNotification(clientStatus, true, null)
@@ -177,8 +183,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When ClientStatus is COMPUTING_STATUS_COMPUTING then expect only one Notification Action with menu_run_mode_disable title`() {
-        val context = ApplicationProvider.getApplicationContext<Context>()
-        val clientStatus = ClientStatus(context, null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
 
         val notification = clientNotification.buildNotification(clientStatus, true, null)
@@ -188,7 +193,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When active is true then expect Notification priority to be high`() {
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         val notification = clientNotification.buildNotification(clientStatus, true, null)
@@ -198,7 +203,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When active is false then expect Notification priority to be low`() {
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         val notification = clientNotification.buildNotification(clientStatus, false, null)
@@ -223,7 +228,7 @@ class ClientNotificationTest {
 
     @Test
     fun `When service is null then expect no exception thrown`() {
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
         assertDoesNotThrow { clientNotification.update(clientStatus, null, true) }
     }
@@ -237,7 +242,7 @@ class ClientNotificationTest {
     fun `When active is true then expect foreground to be true`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.update(clientStatus, monitor, true)
@@ -253,11 +258,10 @@ class ClientNotificationTest {
         val anyBoolean: (Boolean) -> Boolean = { value -> value }
         @Suppress("DEPRECATION")
         justRun { monitor.stopForeground(anyBoolean(any())) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns true
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.update(clientStatus, monitor, false)
@@ -273,11 +277,10 @@ class ClientNotificationTest {
         val anyBoolean: (Boolean) -> Boolean = { value -> value }
         @Suppress("DEPRECATION")
         justRun { monitor.stopForeground(anyBoolean(any())) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns false
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.update(clientStatus, monitor, false)
@@ -292,11 +295,10 @@ class ClientNotificationTest {
         justRun { monitor.startForeground(any(), any()) }
         val anyInteger: (Int) -> Int = { value -> value }
         justRun { monitor.stopForeground(anyInteger(any())) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns true
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.update(clientStatus, monitor, false)
@@ -311,11 +313,10 @@ class ClientNotificationTest {
         justRun { monitor.startForeground(any(), any()) }
         val anyInteger: (Int) -> Int = { value -> value }
         justRun { monitor.stopForeground(anyInteger(any())) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns false
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.update(clientStatus, monitor, false)
@@ -331,11 +332,10 @@ class ClientNotificationTest {
         val anyBoolean: (Boolean) -> Boolean = { value -> value }
         @Suppress("DEPRECATION")
         justRun { monitor.stopForeground(anyBoolean(any())) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns true
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.foreground = true
@@ -352,11 +352,10 @@ class ClientNotificationTest {
         val anyBoolean: (Boolean) -> Boolean = { value -> value }
         @Suppress("DEPRECATION")
         justRun { monitor.stopForeground(anyBoolean(any())) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns false
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.foreground = true
@@ -372,11 +371,10 @@ class ClientNotificationTest {
         justRun { monitor.startForeground(any(), any()) }
         val anyInteger: (Int) -> Int = { value -> value }
         justRun { monitor.stopForeground(anyInteger(any())) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns true
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.foreground = true
@@ -392,11 +390,10 @@ class ClientNotificationTest {
         justRun { monitor.startForeground(any(), any()) }
         val anyInteger: (Int) -> Int = { value -> value }
         justRun { monitor.stopForeground(anyInteger(any())) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns false
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.foreground = true
@@ -409,7 +406,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_NEVER then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.update(clientStatus, monitor, true)
@@ -424,7 +421,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_IDLE then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_IDLE
 
         clientNotification.update(clientStatus, monitor, true)
@@ -439,7 +436,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_SUSPENDED then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_SUSPENDED
 
         clientNotification.update(clientStatus, monitor, true)
@@ -454,7 +451,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_COMPUTING and executingTasks is empty then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
 
         clientNotification.update(clientStatus, monitor, true)
@@ -469,7 +466,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_IDLE then expect status updated and oldActiveTasks to be empty`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_IDLE
 
         clientNotification.update(clientStatus, monitor, true)
@@ -484,7 +481,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_SUSPENDED then expect status updated and oldActiveTasks to be empty`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_SUSPENDED
 
         clientNotification.update(clientStatus, monitor, true)
@@ -499,7 +496,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_NEVER then expect status updated and oldActiveTasks to be empty`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
 
         clientNotification.update(clientStatus, monitor, true)
@@ -514,7 +511,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_COMPUTING and executingTasks is not empty then expect status updated and oldActiveTasks to be equal to executingTasks`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
         clientStatus.setClientStatus(
             CcStatus(),
@@ -541,7 +538,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_COMPUTING and executingTasks contains both running and not running tasks then expect status updated and oldActiveTasks to be equal to executingTasks`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
         clientStatus.setClientStatus(
             CcStatus(),
@@ -571,7 +568,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_COMPUTING and executingTasks contains not running tasks then expect status updated and oldActiveTasks to be empty`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
         clientStatus.setClientStatus(
             CcStatus(),
@@ -597,7 +594,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_COMPUTING, and executingTasks and oldActiveTasks are not empty then expect status updated and oldActiveTasks to be equal to executingTasks`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
         clientStatus.setClientStatus(
             CcStatus(),
@@ -625,7 +622,7 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_COMPUTING, and executingTasks and oldActiveTasks contain different number of records then expect status updated and lists to be equal`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_COMPUTING
         clientStatus.setClientStatus(
             CcStatus(),
@@ -654,11 +651,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_IDLE, active is true and showNotificationDuringSuspend is true then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns true
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_IDLE
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -673,11 +669,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_IDLE, active is true and showNotificationDuringSuspend is false then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns false
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_IDLE
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -692,11 +687,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_IDLE, active is false and showNotificationDuringSuspend is true then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns true
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_IDLE
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -711,11 +705,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_IDLE, active is false and showNotificationDuringSuspend is false then expect notification is hidden`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns false
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_IDLE
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -728,11 +721,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_SUSPENDED, active is true and showNotificationDuringSuspend is true then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns true
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_SUSPENDED
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -747,11 +739,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_SUSPENDED, active is true and showNotificationDuringSuspend is false then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns false
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_SUSPENDED
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -766,11 +757,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_SUSPENDED, active is false and showNotificationDuringSuspend is true then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns true
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_SUSPENDED
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -785,11 +775,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_SUSPENDED, active is false and showNotificationDuringSuspend is false then expect notification is hidden`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns false
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_SUSPENDED
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -802,11 +791,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_NEVER, active is true and showNotificationDuringSuspend is true then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns true
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -821,11 +809,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_NEVER, active is true and showNotificationDuringSuspend is false then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns false
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -840,11 +827,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_NEVER, active is false and showNotificationDuringSuspend is true then expect status updated`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns true
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
         clientNotification.mOldActiveTasks.add(Result())
 
@@ -859,11 +845,10 @@ class ClientNotificationTest {
     fun `When updatedStatus is COMPUTING_STATUS_NEVER, active is false and showNotificationDuringSuspend is false then expect notification is hidden`() {
         val monitor = mockkClass(Monitor::class)
         justRun { monitor.startForeground(any(), any()) }
-        val appPreferences = mockkClass(AppPreferences::class)
         every { monitor.appPreferences } returns appPreferences
         every { appPreferences.showNotificationDuringSuspend } returns false
 
-        val clientStatus = ClientStatus(ApplicationProvider.getApplicationContext(), null, null)
+        val clientStatus = ClientStatus(context, appPreferences, deviceStatus)
         clientStatus.computingStatus = ClientStatus.COMPUTING_STATUS_NEVER
         clientNotification.mOldActiveTasks.add(Result())
 
