@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2023 University of California
+// Copyright (C) 2025 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -15,8 +15,10 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-// check_security.C
+// Functions to check ownership and permissions of files and dirs
+// in the BOINC data directory on Mac OS
 
+//#define DEBUG_CHECK_SECURITY
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -35,8 +37,12 @@
 
 #ifdef __APPLE__                            // If Mac BOINC Manager
 #include "mac_branding.h"
+#include "mac_spawn.h"
 
 bool IsUserInGroup(char* groupName);
+static int check_boinc_users_primarygroupIds(
+    int useFakeProjectUserAndGroup, int isMacInstaller
+);
 #endif
 
 static int CheckNestedDirectories(
@@ -74,18 +80,22 @@ char *bundlePath, char *dataPath,   // These arguments are used only when called
 #endif
 int use_sandbox, int isManager, char* path_to_error, int len
 ) {
-    passwd              *pw;
-    group               *grp;
     char                dir_path[MAXPATHLEN], full_path[MAXPATHLEN];
     struct stat         sbuf;
     int                 retval;
+    int                 retval2;
     int                 useFakeProjectUserAndGroup = 0;
+    int                 isMacInstaller = 0;
 
     useFakeProjectUserAndGroup = ! use_sandbox;
-#ifdef _DEBUG
+#ifdef DEBUG_CHECK_SECURITY
 #ifdef DEBUG_WITH_FAKE_PROJECT_USER_AND_GROUP
     useFakeProjectUserAndGroup = 1;
 #endif
+#ifdef _MAC_INSTALLER
+    isMacInstaller = 1;
+#endif
+
 #if (defined(__APPLE__) &&  ! defined(_MAC_INSTALLER))
     // Debugging Mac client or Mac BOINC Manager
     // Normally, the Mac Development (Debug) builds do not define SANDBOX, so
@@ -102,7 +112,7 @@ int use_sandbox, int isManager, char* path_to_error, int len
         return -1099;
     }
 #endif
-#endif      // _DEBUG
+#endif      // DEBUG_CHECK_SECURITY
 
 // GDB can't attach to applications which are running as a different user or group so
 //  it ignores the S_ISUID and S_ISGID permission bits when launching an application.
@@ -141,9 +151,10 @@ int use_sandbox, int isManager, char* path_to_error, int len
         if ((sbuf.st_mode & (S_ISUID | S_ISGID)) != (S_ISUID | S_ISGID))
             return -1005;
 
-        boinc_master_uid = sbuf.st_uid;
-        boinc_master_gid = sbuf.st_gid;
-
+        if (useFakeProjectUserAndGroup) {
+            boinc_master_uid = sbuf.st_uid;
+            boinc_master_gid = sbuf.st_gid;
+        }
 #ifdef __WXMAC__
     if (!IsUserInGroup(REAL_BOINC_MASTER_NAME))
         return -1099;
@@ -153,53 +164,46 @@ int use_sandbox, int isManager, char* path_to_error, int len
         boinc_master_gid = getegid();
     }
 
+    retval2 = check_boinc_users_primarygroupIds(useFakeProjectUserAndGroup, isMacInstaller);
+
+#ifdef __APPLE__
+    char DataDirPath[MAXPATHLEN];
 #ifdef _MAC_INSTALLER
-   // Require absolute owner and group boinc_master:boinc_master
-    strlcpy(boinc_master_user_name, REAL_BOINC_MASTER_NAME, sizeof(boinc_master_user_name));
-    pw = getpwnam(boinc_master_user_name);
-    if (pw == NULL)
-        return -1006;      // User boinc_master does not exist
-    boinc_master_uid = pw->pw_uid;
-
-    strlcpy(boinc_master_group_name, REAL_BOINC_MASTER_NAME, sizeof(boinc_master_group_name));
-    grp = getgrnam(boinc_master_group_name);
-    if (grp == NULL)
-        return -1007;                // Group boinc_master does not exist
-    boinc_master_gid = grp->gr_gid;
-
-#else   // Use current user and group (see comment above)
-
-    pw = getpwuid(boinc_master_uid);
-    if (pw == NULL)
-        return -1008;      // Should never happen
-    strlcpy(boinc_master_user_name, pw->pw_name, sizeof(boinc_master_user_name));
-
-    grp = getgrgid(boinc_master_gid);
-    if (grp == NULL)
-        return -1009;
-    strlcpy(boinc_master_group_name, grp->gr_name, sizeof(boinc_master_group_name));
-
+    strlcpy(DataDirPath, dataPath, sizeof(dir_path));  // Installer
+#else
+    getcwd(DataDirPath, sizeof(DataDirPath));
 #endif
+    snprintf(full_path, sizeof(full_path),
+        "%s/%s", DataDirPath, FIX_BOINC_USERS_FILENAME
+    );
+    retval = stat(full_path, &sbuf);
+    if (retval)
+    return -1061;
 
-    if (useFakeProjectUserAndGroup) {
-        // For easier debugging of project applications
-        strlcpy(boinc_project_user_name, boinc_master_user_name, sizeof(boinc_project_user_name));
-        strlcpy(boinc_project_group_name, boinc_master_group_name, sizeof(boinc_project_group_name));
-        boinc_project_uid = boinc_master_uid;
-        boinc_project_gid = boinc_master_gid;
-    } else {
-        strlcpy(boinc_project_user_name, REAL_BOINC_PROJECT_NAME, sizeof(boinc_project_user_name));
-        pw = getpwnam(boinc_project_user_name);
-        if (pw == NULL)
-            return -1010;      // User boinc_project does not exist
-        boinc_project_uid = pw->pw_uid;
+    if (sbuf.st_uid != 0)   // root
+         return -1062;
 
-        strlcpy(boinc_project_group_name, REAL_BOINC_PROJECT_NAME, sizeof(boinc_project_group_name));
-        grp = getgrnam(boinc_project_group_name);
-        if (grp == NULL)
-            return -1011;                // Group boinc_project does not exist
-        boinc_project_gid = grp->gr_gid;
+    if (sbuf.st_gid != boinc_master_gid)
+        return -1063;
+
+    if ((sbuf.st_mode & 07777) != 04555)
+        return -1064;
+
+    if (! isMacInstaller) {
+    // MacOS updates often change the PrimaryGroupID of boinc_master and
+    // boinc_project to 20 (staff.)
+        if ((retval2 == -1301) || (retval2 == -1302)) {
+            snprintf(full_path, sizeof(full_path),
+                "\"%s/%s\"", DataDirPath, FIX_BOINC_USERS_FILENAME
+            );
+            printf("Permissions error %d, calling %s\n", retval2, full_path);
+            callPosixSpawn(full_path);  // Try to fix it
+            retval2 = check_boinc_users_primarygroupIds(useFakeProjectUserAndGroup, isMacInstaller);
+        }
     }
+    if (retval2)
+        return retval2;
+#endif
 
 #if 0   // Manager is no longer setgid
 #if (defined(__WXMAC__) || defined(_MAC_INSTALLER)) // If Mac BOINC Manager or installer
@@ -234,15 +238,11 @@ int use_sandbox, int isManager, char* path_to_error, int len
 #endif   // Manager is no longer setgid
 
 
-#ifdef _MAC_INSTALLER
+#if (defined(__WXMAC__) || defined(_MAC_INSTALLER)) // If Mac BOINC Manager or installer
         // Require absolute owner and group boinc_master:boinc_master
         // Get the full path to BOINC Client inside this application's bundle
         //
-        snprintf(full_path, sizeof(full_path),
-            "%s/Contents/Resources/boinc", dir_path
-        );
-
-        retval = stat(full_path, &sbuf);
+        retval = stat("/Applications/BOINCManager.app/Contents/Resources/boinc", &sbuf);
         if (retval)
             return -1016;          // Should never happen
 
@@ -266,7 +266,7 @@ int use_sandbox, int isManager, char* path_to_error, int len
                 snprintf(full_path, sizeof(full_path), "/Library/Screen Savers/%s.saver/Contents/Resources/gfx_switcher", saverName[i]);
                 retval = stat(full_path, &sbuf);
                 if (! retval) {
-#ifdef _DEBUG
+#ifdef DEBUG_CHECK_SECURITY
                     if (sbuf.st_uid != boinc_master_uid)
                         return -1101;
 
@@ -297,13 +297,13 @@ int use_sandbox, int isManager, char* path_to_error, int len
 #ifdef _MAC_INSTALLER
     strlcpy(dir_path, dataPath, sizeof(dir_path));  // Installer
 #else       // _MAC_INSTALLER
-    gid_t               egid = getegid();;
-    uid_t               euid = geteuid();;
+    gid_t               egid = getegid();
+    uid_t               euid = geteuid();
 
     getcwd(dir_path, sizeof(dir_path));             // Client or Manager
 
     if (! isManager) {                                   // If BOINC Client
-    if (egid != boinc_master_gid)
+        if (egid != boinc_master_gid)
             return -1019;     // Client should be running setgid boinc_master
 
         if (euid != boinc_master_uid)
@@ -375,6 +375,26 @@ int use_sandbox, int isManager, char* path_to_error, int len
             return retval;
     }
 
+    snprintf(full_path, sizeof(full_path), "%s/%s", dir_path, PODMAN_DIR);
+    retval = stat(full_path, &sbuf);
+    if (! retval) {                 // Client can create podman directory if it does not yet exist.
+       if (use_sandbox) {
+            if (sbuf.st_gid != boinc_project_gid)
+                return -1071;
+
+            if ((sbuf.st_mode & 0777) != 0770)
+                return -1072;
+        }
+
+        if (sbuf.st_uid != boinc_master_uid)
+            return -1073;
+
+        // Step through slot directories
+        retval = CheckNestedDirectories(full_path, 1, use_sandbox, isManager, false, path_to_error, len);
+        if (retval)
+            return retval;
+    }
+
     snprintf(full_path, sizeof(full_path),
         "%s/%s", dir_path, GUI_RPC_PASSWD_FILE
     );
@@ -482,6 +502,73 @@ int use_sandbox, int isManager, char* path_to_error, int len
 }
 
 
+#ifdef __APPLE__
+static int check_boinc_users_primarygroupIds(int useFakeProjectUserAndGroup, int isMacInstaller) {
+    passwd              *pw;
+    group               *grp;
+
+    if ((!useFakeProjectUserAndGroup) || isMacInstaller) {
+       // Require absolute owner and group boinc_master:boinc_master
+        strlcpy(boinc_master_user_name, REAL_BOINC_MASTER_NAME, sizeof(boinc_master_user_name));
+        pw = getpwnam(boinc_master_user_name);
+        if (pw == NULL)
+            return -1006;      // User boinc_master does not exist
+        boinc_master_uid = pw->pw_uid;
+
+        strlcpy(boinc_master_group_name, REAL_BOINC_MASTER_NAME, sizeof(boinc_master_group_name));
+        grp = getgrnam(boinc_master_group_name);
+        if (grp == NULL)
+            return -1007;                // Group boinc_master does not exist
+        boinc_master_gid = grp->gr_gid;
+
+        // A MacOS update sometimes changes the PrimaryGroupID of user boinc_master to staff (20).
+        if (pw->pw_gid != grp->gr_gid) {
+            return -1301;
+        }
+    } else {    // Use current user and group. See comment near start of check_security()
+                // about "Debugging Mac client or Mac BOINC Manager")
+
+        pw = getpwuid(boinc_master_uid);
+        if (pw == NULL)
+            return -1008;      // Should never happen
+        strlcpy(boinc_master_user_name, pw->pw_name, sizeof(boinc_master_user_name));
+
+        grp = getgrgid(boinc_master_gid);
+        if (grp == NULL)
+            return -1009;
+        strlcpy(boinc_master_group_name, grp->gr_name, sizeof(boinc_master_group_name));
+
+    }
+
+    if (useFakeProjectUserAndGroup) {
+        // For easier debugging of project applications
+        strlcpy(boinc_project_user_name, boinc_master_user_name, sizeof(boinc_project_user_name));
+        strlcpy(boinc_project_group_name, boinc_master_group_name, sizeof(boinc_project_group_name));
+        boinc_project_uid = boinc_master_uid;
+        boinc_project_gid = boinc_master_gid;
+    } else {
+        strlcpy(boinc_project_user_name, REAL_BOINC_PROJECT_NAME, sizeof(boinc_project_user_name));
+        pw = getpwnam(boinc_project_user_name);
+        if (pw == NULL)
+            return -1010;      // User boinc_project does not exist
+        boinc_project_uid = pw->pw_uid;
+
+        strlcpy(boinc_project_group_name, REAL_BOINC_PROJECT_NAME, sizeof(boinc_project_group_name));
+        grp = getgrnam(boinc_project_group_name);
+        if (grp == NULL)
+            return -1011;                // Group boinc_project does not exist
+        boinc_project_gid = grp->gr_gid;
+
+        // A MacOS update sometimes changes the PrimaryGroupID of user boinc_project to staff (20).
+        if (pw->pw_gid != grp->gr_gid) {
+            return -1302;
+        }
+    }
+    return 0;
+}
+#endif
+
+
 static int CheckNestedDirectories(
     char * basepath, int depth,
     int use_sandbox, int isManager,
@@ -529,6 +616,7 @@ static int CheckNestedDirectories(
         if (!S_ISLNK(sbuf.st_mode)) {   // The system ignores ownership & permissions of symbolic links
             if (depth > 1)  {
                 // files and subdirectories created by projects may have owner boinc_master or boinc_project
+                // TODO: Is this is true of subdirectories created by Podman tasks?
                 if ( (sbuf.st_uid != boinc_master_uid) && (sbuf.st_uid != boinc_project_uid) ) {
                     if (!isSlotDir) {
                             retval = -1202;
