@@ -1,6 +1,6 @@
 // This file is part of BOINC.
-// http://boinc.berkeley.edu
-// Copyright (C) 2021 University of California
+// https://boinc.berkeley.edu
+// Copyright (C) 2025 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -20,7 +20,6 @@
 // Try to keep this well-organized and not nested.
 
 #include "version.h"         // version numbers from autoconf
-
 #include "cpp.h"
 #include "config.h"
 
@@ -129,6 +128,7 @@ using std::string;
 using std::min;
 
 #ifdef __APPLE__
+#include "sandbox.h"
 #include <IOKit/IOKitLib.h>
 #include <Carbon/Carbon.h>
 #include <CoreFoundation/CoreFoundation.h>
@@ -1231,10 +1231,171 @@ int HOST_INFO::get_virtualbox_version() {
             pclose(fd);
         }
     }
-
     return 0;
 }
 
+// check if docker/podman is installed and functional on this host.
+// if so, populate docker_version and return true
+//
+bool HOST_INFO::get_docker_version_aux(DOCKER_TYPE type){
+    char cmd[1024], buf[256];
+
+    snprintf(cmd, sizeof(cmd), "%s %s --version 2>/dev/null",
+        docker_cmd_prefix(type),
+        docker_cli_prog(type)
+    );
+    FILE* f = popen(cmd, "r");
+    if (!f) return false;
+    // normally the version is on the first line,
+    // but it's on the 2nd line if using podman-docker
+    //
+    bool found = false;
+    while (fgets(buf, sizeof(buf), f)) {
+        string version;
+        if (get_docker_version_string(type, buf, version)) {
+            safe_strcpy(docker_version, version.c_str());
+            found = true;
+            break;
+        }
+    }
+    pclose(f);
+    if (!found) return false;
+
+#ifdef __APPLE__
+    // download (if not there) and start QEMU VM
+    if (type == PODMAN) {
+        snprintf(cmd, sizeof(cmd),
+            "%s podman machine init 2>/dev/null",
+            docker_cmd_prefix(type)
+        );
+        system(cmd);
+        snprintf(cmd, sizeof(cmd),
+            "%s podman machine start 2>/dev/null",
+            docker_cmd_prefix(type)
+        );
+        system(cmd);
+    }
+#endif
+
+#ifdef __linux__
+    // if we're running as an unprivileged user, Docker/podman may not work.
+    // Check by running the Hello World image.
+    //
+    // Since we do this every time on startup, don't delete the image.
+    //
+    snprintf(cmd, sizeof(cmd),
+        "%s run hello-world 2>/dev/null",
+        docker_cli_prog(type)
+    );
+    found = false;
+    f = popen(cmd, "r");
+    if (f) {
+        while (fgets(buf, sizeof(buf), f)) {
+            if (strstr(buf, "Hello")) {
+                found = true;
+                break;
+            }
+        }
+        pclose(f);
+    }
+    if (!found) {
+        msg_printf(NULL, MSG_INFO,
+            "%s found but 'hello-world' test failed",
+            docker_type_str(type)
+        );
+        docker_version[0] = 0;
+        return false;
+    }
+#endif
+    return true;
+}
+
+bool HOST_INFO::get_docker_version(){
+    bool check_podman = true;
+#ifdef __linux__
+    // podman doesn't work on Linux with remote FS
+    bool remote;
+    int retval = is_filesystem_remote(".", remote);
+    if (!retval && remote) {
+        check_podman = false;
+        msg_printf(NULL, MSG_INFO, "Data dir is remote: not checking podman");
+    }
+#endif
+
+#ifdef __APPLE__
+    // on Mac, Podman requires an accessable dir 'podman'
+    //
+    if (!is_dir(PODMAN_DIR)) {
+        boinc_mkdir(PODMAN_DIR);
+        if (g_use_sandbox) {
+            mode_t old_mask = umask(2); // Allow writing by group
+            chmod(PROJECTS_DIR,
+                S_IRUSR|S_IWUSR|S_IXUSR
+                |S_IRGRP|S_IWGRP|S_IXGRP
+            );
+            umask(old_mask);
+            // Only user boinc_master and group boinc_project can access
+            // project directories, to keep authenticators private
+            set_to_project_group(PODMAN_DIR);
+        }
+    }
+#endif
+
+    if (check_podman) {
+        if (get_docker_version_aux(PODMAN)) {
+            docker_type = PODMAN;
+            return true;
+        }
+    }
+
+    if (get_docker_version_aux(DOCKER)) {
+        docker_type = DOCKER;
+        return true;
+    }
+    return false;
+}
+
+// check if docker compose is installed on this host
+// populate docker_compose_version on success
+//
+bool HOST_INFO::get_docker_compose_version_aux(DOCKER_TYPE type){
+    bool ret = false;
+    char cmd[1024];
+
+    snprintf(cmd, sizeof(cmd),
+        "%s%s compose version 2>/dev/null",
+        docker_cmd_prefix(type),
+        docker_cli_prog(type)
+    );
+    FILE* f = popen(cmd, "r");
+    if (f) {
+        char buf[256];
+        while (fgets(buf, sizeof(buf), f)) {
+            string version;
+            if (get_docker_compose_version_string(type, buf, version)) {
+                safe_strcpy(docker_compose_version, version.c_str());
+                docker_compose_type = type;
+                ret = true;
+                break;
+            }
+        }
+        pclose(f);
+    }
+    return ret;
+}
+
+bool HOST_INFO::get_docker_compose_version(){
+#if defined(__APPLE__) || defined(_WIN32)
+    if (get_docker_compose_version_aux(PODMAN)) {
+        return true;
+    }
+#endif
+    if (get_docker_compose_version_aux(DOCKER)) {
+        return true;
+    }
+    return false;
+}
+
 // get p_vendor, p_model, p_features
 //
 int HOST_INFO::get_cpu_info() {
@@ -1334,6 +1495,12 @@ int HOST_INFO::get_cpu_count() {
     if(cpus_sys_path > p_ncpus){
         p_ncpus = cpus_sys_path;
     }
+#elif __GNU_LIBRARY__ /* glibc */
+    cpu_set_t set;
+
+    if (sched_getaffinity (0, sizeof (set), &set) == 0) {
+        p_ncpus = CPU_COUNT (&set);
+    }
 #elif defined(_SC_NPROCESSORS_ONLN) && !defined(__EMX__) && !defined(__APPLE__)
     // sysconf not working on OS2
     p_ncpus = sysconf(_SC_NPROCESSORS_ONLN);
@@ -1676,6 +1843,9 @@ int HOST_INFO::get_host_info(bool init) {
         get_virtualbox_version();
     }
 
+    get_docker_version();
+    get_docker_compose_version();
+
     get_cpu_info();
     get_cpu_count();
     get_memory_info();
