@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2008 University of California
+// Copyright (C) 2022 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -17,6 +17,8 @@
 
 extern int kill_via_switcher(int pid);
 extern int get_project_gid();
+extern int fix_slot_owners(const int slot);
+extern int fix_owners_in_directory(char* dir_path);
 extern int set_to_project_group(const char* path);
 extern int switcher_exec(const char* util_filename, const char* cmdline);
 extern int client_clean_out_dir(
@@ -27,6 +29,8 @@ extern int remove_project_owned_dir(const char* name);
 extern int remove_project_owned_file_or_dir(const char* path);
 extern int check_security(int use_sandbox, int isManager, char* path_to_error, int len);
 
+#define BOINC_MASTER_USER_NAME "boinc_master"
+#define BOINC_PROJECT_USER_NAME "boinc_project"
 #define BOINC_PROJECT_GROUP_NAME "boinc_project"
 
 extern bool g_use_sandbox;
