@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2023 University of California
+// Copyright (C) 2025 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -107,7 +107,7 @@ public:
     bool                    FireOnRestoreState( wxConfigBase* pConfig );
 
     virtual int             GetListRowCount();
-    void                    FireOnListRender( wxTimerEvent& event );
+    void                    FireOnListRender();
     void                    FireOnListSelected( wxListEvent& event );
     void                    FireOnListDeselected( wxListEvent& event );
     wxString                FireOnListGetItemText( long item, long column ) const;
@@ -155,7 +155,7 @@ protected:
     virtual bool            OnSaveState( wxConfigBase* pConfig );
     virtual bool            OnRestoreState( wxConfigBase* pConfig );
 
-    virtual void            OnListRender( wxTimerEvent& event );
+    virtual void            OnListRender();
     virtual void            OnListSelected( wxListEvent& event );
     virtual void            OnListDeselected( wxListEvent& event );
     virtual void            OnCacheHint(wxListEvent& event);
