@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2022 University of California
+// Copyright (C) 2024 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -29,9 +29,6 @@
 #ifdef __WXMAC__
 #include "mac_util.h"
 #endif
-#ifdef _WIN32
-#include "proc_control.h"
-#endif
 
 #include "BOINCGUIApp.h"
 #include "MainDocument.h"
@@ -43,6 +40,7 @@
 #include "Events.h"
 #include "SkinManager.h"
 #include "version.h"
+#include "DlgGenericMessage.h"
 
 #ifndef _WIN32
 #include <sys/wait.h>
@@ -1298,6 +1296,7 @@ void CMainDocument::CheckForVersionUpdate(bool showMessage) {
     wxString message, title;
     title.Printf(_("Version Update"));
     wxString applicationName = wxGetApp().GetSkinManager()->GetAdvanced()->GetApplicationName();
+    bool newVersionAvailable = false;
     if (IsConnected()) {
         rpc.get_newer_version(version, url);
 
@@ -1305,7 +1304,8 @@ void CMainDocument::CheckForVersionUpdate(bool showMessage) {
             return;
 
         if (!version.empty() && !url.empty()) {
-            message.Printf(_("A new version of %s is available. You can download it here: %s"), applicationName, url);
+            message.Printf(_("A new version of %s is available.\nYou can download it here: %s"), applicationName, url);
+            newVersionAvailable = true;
         }
         else {
             message.Printf(_("There is no new version of %s available for download."), applicationName);
@@ -1315,7 +1315,16 @@ void CMainDocument::CheckForVersionUpdate(bool showMessage) {
         message.Printf(_("%s is not connected to the client"), applicationName);
     }
     if (showMessage) {
-        wxGetApp().SafeMessageBox(message, title);
+        CDlgGenericMessageParameters params;
+        params.caption = title;
+        params.message = message;
+        params.showDisableMessage = false;
+        params.button1 = CDlgGenericMessageButton(newVersionAvailable, wxID_OK, _("Go to download page"));
+        params.button2 = CDlgGenericMessageButton(true, wxID_CANCEL, _("Close"));
+        CDlgGenericMessage dlg(wxGetApp().GetFrame(), &params);
+        if (dlg.ShowModal() == wxID_OK) {
+            wxLaunchDefaultBrowser(url);
+        }
     }
 }
 
@@ -1561,7 +1570,7 @@ int CMainDocument::GetWorkCount() {
 }
 
 
-int CMainDocument::WorkSuspend(char* url, char* name) {
+int CMainDocument::WorkSuspend(const char* url, const char* name) {
     int iRetVal = 0;
 
     RESULT* pStateResult = state.lookup_result(url, name);
@@ -1575,7 +1584,7 @@ int CMainDocument::WorkSuspend(char* url, char* name) {
 }
 
 
-int CMainDocument::WorkResume(char* url, char* name) {
+int CMainDocument::WorkResume(const char* url, const char* name) {
     int iRetVal = 0;
 
     RESULT* pStateResult = state.lookup_result(url, name);
@@ -1630,10 +1639,27 @@ RUNNING_GFX_APP* CMainDocument::GetRunningGraphicsApp(RESULT* rp) {
             }
 
             // Graphics app is still running but the slot now has a different task
-            KillGraphicsApp((*gfx_app_iter).pid);
-        }
+#ifdef __WXMAC__
+            // For some graphics apps (including Einstein), we must explicitly delete both
+            // our forked process and its child graphics app, or the other will remain. A
+            // race condition can occur because of the time it takes for our forked process
+            // to launch the graphics app, so we periodically poll for the child's PID until
+            // it is available. If we don't have it yet, we defer killing our forked process.
+            if (g_use_sandbox) {
+                if (!GetGFXPIDFromForkedPID(&(*gfx_app_iter))) {
+                    return 0;    // Ignore "Stop graphics" button until we have graphics app's pid
+                }
+                KillGraphicsApp((*gfx_app_iter).gfx_pid);
+           }
+#endif
+        KillGraphicsApp((*gfx_app_iter).pid);
+       }
 
         // Either the graphics app had already exited or we just killed it
+#ifdef __WXMAC__
+        // Graphics app wrote files in slot directory as logged in user, not boinc_master
+        fix_slot_file_owners((*gfx_app_iter).slot);
+#endif
         (*gfx_app_iter).name.clear();
         (*gfx_app_iter).project_url.clear();
         m_running_gfx_apps.erase(gfx_app_iter);
@@ -1672,7 +1698,25 @@ void CMainDocument::KillInactiveGraphicsApps() {
         }
 
         if (!bStillRunning) {
+#ifdef __WXMAC__
+            // For some graphics apps (including Einstein), we must explicitly delete both
+            // our forked process and its child graphics app, or the other will remain. A
+            // race condition can occur because of the time it takes for our forked process
+            // to launch the graphics app, so we periodically poll for the child's PID until
+            // it is available. If we don't have it yet, we defer killing our forked process.
+            if (g_use_sandbox) {
+                if (!GetGFXPIDFromForkedPID(&(*gfx_app_iter))) {
+                    return;    // Ignore "Stop graphics" button until we have graphics app's pid
+                }
+                KillGraphicsApp((*gfx_app_iter).gfx_pid);
+                KillGraphicsApp((*gfx_app_iter).pid);
+
+                // Graphics app wrote files in slot directory as logged in user, not boinc_master
+                fix_slot_file_owners((*gfx_app_iter).slot);
+           }
+#else
             KillGraphicsApp((*gfx_app_iter).pid);
+#endif
             gfx_app_iter = m_running_gfx_apps.erase(gfx_app_iter);
         } else {
             gfx_app_iter++;
@@ -1682,6 +1726,7 @@ void CMainDocument::KillInactiveGraphicsApps() {
 }
 
 
+// KillAllRunningGraphicsApps() is called only from our destructor
 void CMainDocument::KillAllRunningGraphicsApps()
 {
     size_t i, n;
@@ -1690,7 +1735,30 @@ void CMainDocument::KillAllRunningGraphicsApps()
     n = m_running_gfx_apps.size();
     for (i=0; i<n; i++) {
         gfx_app_iter = m_running_gfx_apps.begin();
+#ifdef __WXMAC__
+        // For some graphics apps (including Einstein), we must explicitly delete both
+        // our forked process and its child graphics app, or the other will remain. A
+        // race condition can occur because of the time it takes for our forked process
+        // to launch the graphics app, so we periodically poll for the child's PID until
+        // it is available. If we don't have it yet, we defer killing our forked process.
+        if (g_use_sandbox) {
+            for (int j=0; j<100; ++j) { // Wait 1 second max for gfx app's pid
+                if (GetGFXPIDFromForkedPID(&(*gfx_app_iter))) {
+                    KillGraphicsApp((*gfx_app_iter).gfx_pid);
+                    break;
+                }
+                boinc_sleep(.01);
+            }
+            KillGraphicsApp((*gfx_app_iter).pid);
+
+            // Graphics app wrote files in slot directory as logged in user, not boinc_master
+            fix_slot_file_owners((*gfx_app_iter).slot);
+        } else {
+            KillGraphicsApp((*gfx_app_iter).pid);
+        }
+#else
         KillGraphicsApp((*gfx_app_iter).pid);
+#endif
         (*gfx_app_iter).name.clear();
         (*gfx_app_iter).project_url.clear();
         m_running_gfx_apps.erase(gfx_app_iter);
@@ -1700,39 +1768,59 @@ void CMainDocument::KillAllRunningGraphicsApps()
 
 #ifdef _WIN32
 void CMainDocument::KillGraphicsApp(HANDLE pid) {
-    kill_program(pid);
+    kill_process(pid);
 }
 #else
 void CMainDocument::KillGraphicsApp(int pid) {
-    char* argv[6];
-    char currentDir[1024];
-    char thePIDbuf[20];
-    int id;
-
-
-    if (g_use_sandbox) {
-        snprintf(thePIDbuf, sizeof(thePIDbuf), "%d", pid);
-        argv[0] = "switcher";
-        argv[1] = "/bin/kill";
-        argv[2] =  "kill";
-        argv[3] = "-KILL";
-        argv[4] = thePIDbuf;
-        argv[5] = 0;
-
-        run_program(
-            getcwd(currentDir, sizeof(currentDir)),
-            "./switcher/switcher",
-            5,
-            argv,
-            0,
-            id
-        );
-    } else {
-        kill_program(pid);
-    }
+    // As of MacOS 13.0 Ventura IOSurface cannot be used to share graphics
+    // between apps unless they are running as the same user, so we no
+    // longer run the graphics apps as user boinc_master.
+    kill_process(pid);
 }
 #endif
 
+
+#ifdef __WXMAC__
+// For some graphics apps (including Einstein), we must explicitly delete both
+// our forked process and its child graphics app, or the other will remain. A
+// race condition can occur because of the time it takes for our forked process
+// to launch the graphics app, so we periodically call this method to poll for
+// the child's PID until it is available..
+//
+int CMainDocument::GetGFXPIDFromForkedPID(RUNNING_GFX_APP* gfx_app) {
+    char buf[256];
+
+    if (gfx_app->gfx_pid) return gfx_app->gfx_pid;    // We already found it
+
+    // The graphics app is the child of our forked process. Get its PID
+    FILE *fp = (FILE*)popen("ps -xoppid,pid","r");
+    fgets(buf, sizeof(buf), fp);    // Skip the header line
+    int parent, child;
+
+    // Find the process whose parent is our forked process
+    while (fscanf(fp, "%d %d\n", &parent, &child) == 2) {
+        if (parent == gfx_app->pid) {
+            gfx_app->gfx_pid = child;
+            break;
+        }
+    }
+    pclose(fp);
+    return gfx_app->gfx_pid;
+}
+
+
+int CMainDocument::fix_slot_file_owners(int slot) {
+    if (g_use_sandbox) {
+        // Graphics apps run by Manager write files in slot directory
+        // as logged in user, not boinc_master. This ugly hack tells
+        // BOINC client to fix all ownerships in this slot directory
+        rpcClient.run_graphics_app("stop", slot, "");
+    }
+    return 0;
+}
+#endif
+
+
 int CMainDocument::WorkShowGraphics(RESULT* rp) {
     int iRetVal = 0;
 
@@ -1750,67 +1838,94 @@ int CMainDocument::WorkShowGraphics(RESULT* rp) {
 #ifdef __WXMSW__
         HANDLE   id;
 #else
-        int      id;
+        int      id = 0;
 #endif
 
         // See if we are already running the graphics application for this task
         previous_gfx_app = GetRunningGraphicsApp(rp);
 
         if (previous_gfx_app) {
-            // If graphics app is already running, the button has changed to
-            // "Stop graphics", so we end the graphics app.
-            //
+#ifdef __WXMAC__
+            if (g_use_sandbox) {
+                // If graphics app is already running, the button has changed to
+                // "Stop graphics", so we end the graphics app.
+                //
+                // For some graphics apps (including Einstein), we must explicitly delete both
+                // our forked process and its child graphics app, or the other will remain. A
+                // race condition can occur because of the time it takes for our forked process
+                // to launch the graphics app, so we periodically poll for the child's PID until
+                // it is available. If we don't have it yet, then defer responding to the "Stop
+                // graphics" button.
+                if (!GetGFXPIDFromForkedPID(previous_gfx_app)) {
+                    return 0;    // Ignore "Stop graphics" button until we have graphics app's pid
+                }
+                KillGraphicsApp(previous_gfx_app->gfx_pid);
+            }
+#endif
             KillGraphicsApp(previous_gfx_app->pid); // User clicked on "Stop graphics" button
+            GetRunningGraphicsApp(rp);  // Let GetRunningGraphicsApp() do necessary clean up
             return 0;
         }
 
-#ifndef __WXMSW__
-        char* argv[4];
-
-        argv[0] = "switcher";
-        // For unknown reasons on Macs, the graphics application
-        // exits with "RegisterProcess failed (error = -50)" unless
-        // we pass its full path twice in the argument list to execv.
-        //
-        argv[1] = (char *)rp->graphics_exec_path;
-        argv[2] = (char *)rp->graphics_exec_path;
-        argv[3] = 0;
-
-        if (g_use_sandbox) {
-            iRetVal = run_program(
-                rp->slot_path,
-                "../../switcher/switcher",
-                3,
-                argv,
-                0,
-                id
-            );
-        } else {
-            iRetVal = run_program(
-                rp->slot_path,
-                rp->graphics_exec_path,
-                1,
-                &argv[2],
-                0,
-                id
-            );
-        }
-#else
-        // If graphics app is already running, don't launch a second instance
-        //
-        if (previous_gfx_app) return 0;
 
         char* argv[2] = {
             rp->graphics_exec_path,
             NULL
         };
 
+#ifdef __WXMAC__
+        if (g_use_sandbox) {    // Used only by Mac
+            int pid = fork();
+            char path[MAXPATHLEN];
+            char cmd[2048];
+
+            // As of MacOS 13.0 Ventura IOSurface cannot be used to share graphics
+            // between apps unless they are running as the same user, so we no
+            // longer run the graphics apps as user boinc_master. To replace the
+            // security that was provided by running as a different user, we use
+            // sandbox-exec() to launch the graphics apps. Note that sandbox-exec()
+            // is marked deprecated because it is an Apple private API so the syntax
+            // of the security specifications is subject to change without notice.
+            // But it is used widely in Apple's software, and the security profile
+            // elements we use are very unlikely to change.
+            if (pid == 0) {
+                // For unknown reasons, the graphics application exits with
+                // "RegisterProcess failed (error = -50)" unless we pass its
+                // full path twice in the argument list to execv.
+                strlcpy(cmd, "sandbox-exec -f \"", sizeof(cmd));
+                getPathToThisApp(path, sizeof(path));
+                strlcat(cmd, path, sizeof(cmd)); // path to this executable
+                strlcat(cmd, "/Contents/Resources/mac_restrict_access.sb\" \"", sizeof(cmd)); // path to sandboxing profile
+                strlcat(cmd, rp->graphics_exec_path, sizeof(cmd)); // path to graphics app
+                strlcat(cmd, "\"", sizeof(cmd)); // path to sandboxing profile
+                chdir(rp->slot_path);
+                iRetVal = callPosixSpawn(cmd);
+
+                exit(0);
+            }
+            id = pid;
+            // The graphics app is the child of our forked process. Get its PID
+            gfx_app.pid = id;
+            gfx_app.gfx_pid = 0;    // Initialize GetGFXPIDFromForkedPID()
+            gfx_app.gfx_pid = GetGFXPIDFromForkedPID(&gfx_app);
+        } else  // !g_use_sandbox
+#endif  // __WXMAC__
+#ifndef __WXMSW__
+        {
+            iRetVal = run_program(
+                rp->slot_path,
+                rp->graphics_exec_path,
+                1,
+                argv,
+                id
+            );
+        }
+#else
         iRetVal = run_program(
             rp->slot_path,
             rp->graphics_exec_path,
             1,
             argv,
-            0,
             id
         );
 #endif
@@ -1842,7 +1957,19 @@ int CMainDocument::WorkShowVMConsole(RESULT* res) {
         wxExecute(strCommand);
 #elif defined(__WXGTK__)
         strCommand = wxT("rdesktop-vrdp ") + strConnection;
-        wxExecute(strCommand);
+        int pid = wxExecute(strCommand);
+        // newer versions of VirtualBox don't include rdesktop-vrdp;
+        // try a standard version instead
+        //
+        if (pid == 0) {
+            strCommand = wxT("rdesktop ") + strConnection;
+            pid = wxExecute(strCommand);
+        }
+        if (pid == 0) {
+            strCommand = wxT("xfreerdp ") + strConnection;
+            pid = wxExecute(strCommand);
+        }
+        // show an error if all the above fail?
 #elif defined(__WXMAC__)
         OSStatus status = noErr;
         char pathToCoRD[MAXPATHLEN];
@@ -1876,7 +2003,7 @@ int CMainDocument::WorkShowVMConsole(RESULT* res) {
 }
 
 
-int CMainDocument::WorkAbort(char* url, char* name) {
+int CMainDocument::WorkAbort(const char* url, const char* name) {
     int iRetVal = 0;
 
     RESULT* pStateResult = state.lookup_result(url, name);
@@ -2645,7 +2772,7 @@ static void hsv2rgb(
 //
 void color_cycle(int i, int n, wxColour& color) {
     double h = (double)i/(double)n;
-    double r, g, b;
+    double r = 0.0, g = 0.0, b = 0.0;
     double v = .75;
     if (n > 6) v = .6 + (i % 3)*.1;
         // cycle through 3 different brightnesses
@@ -2660,10 +2787,8 @@ wxString FormatTime(double secs) {
     if (secs <= 0) {
         return wxT("---");
     }
-    wxInt32 iHour = (wxInt32)(secs / (60 * 60));
-    wxInt32 iMin  = (wxInt32)(secs / 60) % 60;
-    wxInt32 iSec  = (wxInt32)(secs) % 60;
-    wxTimeSpan ts = wxTimeSpan(iHour, iMin, iSec);
+
+    wxTimeSpan ts = convert_to_timespan(secs);
     return ts.Format((secs>=86400)?"%Dd %H:%M:%S":"%H:%M:%S");
 }
 
@@ -2672,6 +2797,14 @@ wxString format_number(double x, int nprec) {
 
 }
 
+wxTimeSpan convert_to_timespan(double secs) {
+    wxInt32 iHour = (wxInt32)(secs / (60 * 60));
+    wxInt32 iMin  = (wxInt32)(secs / 60) % 60;
+    wxInt32 iSec  = (wxInt32)(secs) % 60;
+    wxTimeSpan ts = wxTimeSpan(iHour, iMin, iSec);
+    return (ts);
+}
+
 // the autoattach process deletes the installer filename file when done
 //
 bool autoattach_in_progress() {
