@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2022 University of California
+// Copyright (C) 2024 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -136,7 +136,6 @@ NSInteger myWindowNumber;
 NSRect gMovingRect;
 float gImageXIndent;
 float gTextBoxHeight;
-CGFloat gActualTextBoxHeight;
 NSPoint gCurrentPosition;
 NSPoint gCurrentDelta;
 
@@ -151,6 +150,7 @@ static pid_t childPid;
 static int gfxAppWindowNum;
 static NSView *imageView;
 static char gfxAppPath[MAXPATHLEN];
+static char gfxWuName[MAXPATHLEN];
 static int taskSlot;
 static NSRunningApplication *childApp;
 static double gfxAppStartTime;
@@ -176,12 +176,17 @@ static bool myIsPreview;
 #define MAXWAITFORCONNECTIONCATALINA 12.0
 #define MAX_CGWINDOWLIST_TRIES 3
 
+#define MAJORBOINCGFXLIBNEEDEDFORSONOMA 7
+#define MINORBOINCGFXLIBNEEDEDFORSONOMA 24
+#define RELEASEBOINCGFXLIBNEEDEDFORSONOMA 4
+
 int signof(float x) {
     return (x > 0.0 ? 1 : -1);
 }
 
-void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
+void launchedGfxApp(char * appPath, char * wuName, pid_t thePID, int slot) {
     strlcpy(gfxAppPath, appPath, sizeof(gfxAppPath));
+    strlcpy(gfxWuName, wuName, sizeof(gfxWuName));
     childPid = thePID;
     taskSlot = slot;
     gfxAppStartTime = getDTime();
@@ -189,12 +194,24 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
     if (thePID == 0) {
         useCGWindowList = false;
         gfxAppStartTime = 0.0;
+        gfxWuName[0] = '\0';
+        if (mySharedGraphicsController) {
+            [ mySharedGraphicsController cleanUpOpenGL ];
+        }
         if (imageView) {
             // removeFromSuperview must be called from main thread
-            if (pthread_equal(mainThreadID, pthread_self())) {
+            if (NSThread.isMainThread) {
                 [imageView removeFromSuperview];   // Releases imageView
-                imageView = nil;
+            } else {
+                dispatch_sync(dispatch_get_main_queue(), ^{
+                    [imageView removeFromSuperview];   // Releases imageView
+                });
             }
+            imageView = nil;
+        }
+    } else {    // thePID != 0
+        if (childPid) {
+            childApp = [NSRunningApplication runningApplicationWithProcessIdentifier:childPid];
         }
     }
 }
@@ -214,6 +231,7 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
     gIsMojave = (compareOSVersionTo(10, 14) >= 0);
     gIsCatalina = (compareOSVersionTo(10, 15) >= 0);
     gIsBigSur = (compareOSVersionTo(11, 0) >= 0);
+    gIsSonoma = (compareOSVersionTo(14, 0) >= 0);
 
     if (gIsCatalina) {
         // Under OS 10.15, isPreview is often true even when it shouldn't be
@@ -354,8 +372,6 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
         gCurrentDelta.x = 1.0;
         gCurrentDelta.y = 1.0;
 
-        gActualTextBoxHeight = MINTEXTBOXHEIGHT;
-
         [ self setAnimationTimeInterval:1/8.0 ];
     }
 
@@ -406,7 +422,13 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
     if (imageView) {
         useCGWindowList = false;
         // removeFromSuperview must be called from main thread
-        [imageView removeFromSuperview];   // Releases imageView
+        if (NSThread.isMainThread) {
+            [imageView removeFromSuperview];   // Releases imageView
+        } else {
+            dispatch_sync(dispatch_get_main_queue(), ^{
+                [imageView removeFromSuperview];   // Releases imageView
+            });
+        }
         imageView = nil;
     }
 
@@ -447,12 +469,32 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
     NSUInteger n;
     double maxWaitTime;
     NSRect currentDrawingRect, eraseRect;
+    CGFloat actualTextBoxHeight = MINTEXTBOXHEIGHT;
     NSPoint imagePosition;
     char *msg;
     CFStringRef cf_msg;
     double timeToBlock, frameStartTime = getDTime();
     HIThemeTextInfo textInfo;
 
+    NSWindow *myWindow = [ self window ];
+    NSRect windowFrame = [ myWindow frame ];
+
+    if (gIsSonoma) {
+        // Under MacOS 14 Sonoma, screensavers continue to run and "draw" invisibly
+        // after they are dismissed by user activity, to allow them to be used as
+        // wallpaper. Since we don't want the BOINC screensaver to be used as wallpaper,
+        // this would waste system resources.
+        // The only way I've found to determine that the screensaver has been dismissed
+        // by the user is this test of the window level.
+        if ((windowFrame.size.width > 500.) && (windowFrame.size.height > 500.)) {
+            if ([ [ self window ] level ] == 0) {
+//TODO: more cleanup as in stopAnimation ??
+                closeBOINCSaver();
+                return;
+            }
+        }
+    }
+
    if (myIsPreview) {
 #if 1   // Currently drawRect just draws our logo in the preview window
         if (gPreview_Image == NULL) {
@@ -474,8 +516,6 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
         return;
     }
 
-    NSWindow *myWindow = [ self window ];
-
 #if ! DEBUG_UNDER_XCODE
     // For unkown reasons, OS 10.7 Lion screensaver and later delay several seconds
     // after user activity before calling stopAnimation, so we check user activity here
@@ -489,7 +529,6 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
         }
     }
 
-    NSRect windowFrame = [ myWindow frame ];
     if ( (windowFrame.origin.x != 0) || (windowFrame.origin.y != 0) ) {
         // Hide window on second display to aid in debugging
 #ifdef _DEBUG
@@ -544,7 +583,7 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
         //
         if (childApp) {
              if (![ childApp activateWithOptions:NSApplicationActivateIgnoringOtherApps ]) {
-                launchedGfxApp("", 0, -1);  // Graphics app is no longer running
+                launchedGfxApp("", "", 0, -1);  // Graphics app is no longer running
              } else if (useCGWindowList) {
                 // As a safety precaution, prevent terminating gfx app while copying its window
                 pthread_mutex_lock(&saver_mutex);
@@ -592,7 +631,13 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
 
     if (imageView && !useCGWindowList) {
         // removeFromSuperview must be called from main thread
-        [imageView removeFromSuperview];   // Releases imageView
+        if (NSThread.isMainThread) {
+            [imageView removeFromSuperview];   // Releases imageView
+        } else {
+            dispatch_sync(dispatch_get_main_queue(), ^{
+                [imageView removeFromSuperview];   // Releases imageView
+            });
+        }
         imageView = nil;
     }
 
@@ -612,16 +657,26 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
             maxWaitTime = gIsCatalina ? MAXWAITFORCONNECTIONCATALINA : MAXWAITFORCONNECTION;
             if ((getDTime() - gfxAppStartTime) > maxWaitTime) {
                 if (gIsCatalina) {
+                    if (gMach_bootstrap_unavailable_to_screensavers) {
+                        // Is the gfx app built with a new enough BOINC graphics library version?
+                        if (compareBOINCLibVersionTo(MAJORBOINCGFXLIBNEEDEDFORSONOMA, MINORBOINCGFXLIBNEEDEDFORSONOMA, RELEASEBOINCGFXLIBNEEDEDFORSONOMA) >= 0) {
+                            runningSharedGraphics = true;
+                            if (childPid) {
+                                gfxAppStartTime = 0.0;
+                                childApp = [NSRunningApplication runningApplicationWithProcessIdentifier:childPid];
+                            }
+                        }
+                    }
                     // CGWindowListCopyWindowInfo and CGWindowListCreateImage can copy
                     // windows between user boinc_project and the user running the
                     // screensaver only if OS < 10.15 (before Catalina)
-                    incompatibleGfxApp(gfxAppPath, childPid, taskSlot);
+                    incompatibleGfxApp(gfxAppPath, gfxWuName, childPid, taskSlot);
                 } else {
                     if (++CGWindowListTries > MAX_CGWINDOWLIST_TRIES) {
                         // After displaying message for 5 seconds, incompatibleGfxApp
                         // will call launchedGfxApp("", 0, -1) which will clear
                         // gfxAppStartTime and CGWindowListTries
-                        incompatibleGfxApp(gfxAppPath, childPid, taskSlot);
+                        incompatibleGfxApp(gfxAppPath, gfxWuName, childPid, taskSlot);
                     } else {
                         if ([self setUpToUseCGWindowList]) {
                             CGWindowListTries = 0;
@@ -688,30 +743,18 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
 
     if ( (msg != NULL) && (msg[0] != '\0') ) {
 
-        // Set direction of motion to "bounce" off edges of screen
-       if (currentDrawingRect.origin.x <= SAFETYBORDER) {
-            gCurrentDelta.x = (float)SSRandomIntBetween(MINDELTA, MAXDELTA) / 16.;
-            gCurrentDelta.y = (float)(SSRandomIntBetween(MINDELTA, MAXDELTA) * signof(gCurrentDelta.y)) / 16.;
-        }
-        if ( (currentDrawingRect.origin.x + currentDrawingRect.size.width) >=
-                    (viewBounds.origin.x + viewBounds.size.width - SAFETYBORDER) ) {
-            gCurrentDelta.x = -(float)SSRandomIntBetween(MINDELTA, MAXDELTA) / 16.;
-            gCurrentDelta.y = (float)(SSRandomIntBetween(MINDELTA, MAXDELTA) * signof(gCurrentDelta.y)) / 16.;
-        }
-        if (currentDrawingRect.origin.y + gTextBoxHeight - gActualTextBoxHeight <= SAFETYBORDER) {
-            gCurrentDelta.y = (float)SSRandomIntBetween(MINDELTA, MAXDELTA) / 16.;
-            gCurrentDelta.x = (float)(SSRandomIntBetween(MINDELTA, MAXDELTA) * signof(gCurrentDelta.x)) / 16.;
-        }
-        if ( (currentDrawingRect.origin.y + currentDrawingRect.size.height) >=
-                   (viewBounds.origin.y + viewBounds.size.height - SAFETYBORDER) ) {
-            gCurrentDelta.y = -(float)SSRandomIntBetween(MINDELTA, MAXDELTA) / 16.;
-            gCurrentDelta.x = (float)(SSRandomIntBetween(MINDELTA, MAXDELTA) * signof(gCurrentDelta.x)) / 16.;
-        }
-#if 0
-        // For testing
-        gCurrentDelta.x = 0;
-        gCurrentDelta.y = 0;
-#endif
+        cf_msg = CFStringCreateWithCString(NULL, msg, kCFStringEncodingMacRoman);
+
+        CTFontRef myFont = CTFontCreateWithName(CFSTR("Helvetica"), 20, NULL);
+        HIThemeTextInfo theTextInfo = {kHIThemeTextInfoVersionOne, kThemeStateActive, kThemeSpecifiedFont,
+                    kHIThemeTextHorizontalFlushLeft, kHIThemeTextVerticalFlushTop,
+                    kHIThemeTextBoxOptionNone, kHIThemeTextTruncationNone, 0, false,
+                    0, myFont
+                    };
+        textInfo = theTextInfo;
+
+        HIThemeGetTextDimensions(cf_msg, (float)gMovingRect.size.width, &textInfo, NULL, &actualTextBoxHeight, NULL);
+        gTextBoxHeight = actualTextBoxHeight + TEXTBOXTOPBORDER;
 
         if (!isErased) {
             [[NSColor blackColor] set];
@@ -751,7 +794,34 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
             eraseRect = NSInsetRect(eraseRect, -1, -1);
             NSRectFill(eraseRect);
 
-            isErased  = true;
+            isErased = true;
+           // If text has changed and it now goes below bottom of screen, jump up to show it all.
+            if ((gCurrentPosition.y - gTextBoxHeight) <= SAFETYBORDER) {
+                gCurrentPosition.y = SAFETYBORDER + 1 + gTextBoxHeight;
+                if (gCurrentDelta.y < 0) {
+                    gCurrentDelta.y = (float)SSRandomIntBetween(MINDELTA, MAXDELTA) / 16.;
+                }
+            }
+
+            // Set direction of motion to "bounce" off edges of screen
+           if ( (gCurrentDelta.x < 0) && (gCurrentPosition.x <= SAFETYBORDER) ) {
+                gCurrentDelta.x = (float)SSRandomIntBetween(MINDELTA, MAXDELTA) / 16.;
+                gCurrentDelta.y = (float)(SSRandomIntBetween(MINDELTA, MAXDELTA) * signof(gCurrentDelta.y)) / 16.;
+            }
+            if ( (gCurrentDelta.x > 0) && ( (gCurrentPosition.x + gMovingRect.size.width) >=
+                        (viewBounds.origin.x + viewBounds.size.width - SAFETYBORDER) ) ){
+                gCurrentDelta.x = -(float)SSRandomIntBetween(MINDELTA, MAXDELTA) / 16.;
+                gCurrentDelta.y = (float)(SSRandomIntBetween(MINDELTA, MAXDELTA) * signof(gCurrentDelta.y)) / 16.;
+            }
+            if ( (gCurrentDelta.y < 0) && (gCurrentPosition.y - gTextBoxHeight <= SAFETYBORDER) ) {
+                gCurrentDelta.y = (float)SSRandomIntBetween(MINDELTA, MAXDELTA) / 16.;
+                gCurrentDelta.x = (float)(SSRandomIntBetween(MINDELTA, MAXDELTA) * signof(gCurrentDelta.x)) / 16.;
+            }
+            if ( (gCurrentDelta.y > 0) && ( (gCurrentPosition.y + gMovingRect.size.height) >=
+                       (viewBounds.origin.y + viewBounds.size.height - SAFETYBORDER) ) ) {
+                gCurrentDelta.y = -(float)SSRandomIntBetween(MINDELTA, MAXDELTA) / 16.;
+                gCurrentDelta.x = (float)(SSRandomIntBetween(MINDELTA, MAXDELTA) * signof(gCurrentDelta.x)) / 16.;
+            }
         }
 
         // Get the new drawing area
@@ -762,48 +832,33 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
         imagePosition.y = (float) (int)gCurrentPosition.y;
 
         [ gBOINC_Logo drawAtPoint:imagePosition fromRect:NSZeroRect operation:NSCompositeCopy fraction:1.0 ];
+#if 0
+    // For testing
+    gCurrentDelta.x = 0;
+    gCurrentDelta.y = 0;
+#endif
+        CGRect bounds = CGRectMake((float) ((int)gCurrentPosition.x),
+                             viewBounds.size.height - imagePosition.y + TEXTBOXTOPBORDER,
+                             gMovingRect.size.width,
+                             MAXTEXTBOXHEIGHT
+                        );
 
-        if ( (msg != NULL) && (msg[0] != '\0') ) {
-            cf_msg = CFStringCreateWithCString(NULL, msg, kCFStringEncodingMacRoman);
+        CGContextSaveGState (myContext);
+        CGContextTranslateCTM (myContext, 0, viewBounds.origin.y + viewBounds.size.height);
+        CGContextScaleCTM (myContext, 1.0f, -1.0f);
 
-            CGRect bounds = CGRectMake((float) ((int)gCurrentPosition.x),
-                                 viewBounds.size.height - imagePosition.y + TEXTBOXTOPBORDER,
-                                 gMovingRect.size.width,
-                                 MAXTEXTBOXHEIGHT
-                            );
+        CGFloat myWhiteComponents[] = {1.0, 1.0, 1.0, 1.0};
+        CGColorSpaceRef myColorSpace = CGColorSpaceCreateDeviceRGB ();
+        CGColorRef myTextColor = CGColorCreate(myColorSpace, myWhiteComponents);
 
-            CGContextSaveGState (myContext);
-            CGContextTranslateCTM (myContext, 0, viewBounds.origin.y + viewBounds.size.height);
-            CGContextScaleCTM (myContext, 1.0f, -1.0f);
+        CGContextSetFillColorWithColor(myContext, myTextColor);
 
-            CTFontRef myFont = CTFontCreateWithName(CFSTR("Helvetica"), 20, NULL);
+        HIThemeDrawTextBox(cf_msg, &bounds, &textInfo, myContext, kHIThemeOrientationNormal);
 
-            HIThemeTextInfo theTextInfo = {kHIThemeTextInfoVersionOne, kThemeStateActive, kThemeSpecifiedFont,
-                        kHIThemeTextHorizontalFlushLeft, kHIThemeTextVerticalFlushTop,
-                        kHIThemeTextBoxOptionNone, kHIThemeTextTruncationNone, 0, false,
-                        0, myFont
-                        };
-            textInfo = theTextInfo;
-
-            HIThemeGetTextDimensions(cf_msg, (float)gMovingRect.size.width, &textInfo, NULL, &gActualTextBoxHeight, NULL);
-            gActualTextBoxHeight += TEXTBOXTOPBORDER;
-
-            CGFloat myWhiteComponents[] = {1.0, 1.0, 1.0, 1.0};
-            CGColorSpaceRef myColorSpace = CGColorSpaceCreateDeviceRGB ();
-            CGColorRef myTextColor = CGColorCreate(myColorSpace, myWhiteComponents);
-
-            CGContextSetFillColorWithColor(myContext, myTextColor);
-
-            HIThemeDrawTextBox(cf_msg, &bounds, &textInfo, myContext, kHIThemeOrientationNormal);
-
-            CGColorRelease(myTextColor);
-            CGColorSpaceRelease(myColorSpace);
-            CGContextRestoreGState (myContext);
-            CFRelease(cf_msg);
-        }
-
-        gTextBoxHeight = MAXTEXTBOXHEIGHT + TEXTBOXTOPBORDER;
-        gMovingRect.size.height = [gBOINC_Logo size].height + gTextBoxHeight;
+        CGColorRelease(myTextColor);
+        CGColorSpaceRelease(myColorSpace);
+        CGContextRestoreGState (myContext);
+        CFRelease(cf_msg);
 
         isErased  = false;
 
@@ -813,7 +868,6 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
             [[NSColor blackColor] set];
             isErased  = true;
             NSRectFill(eraseRect);
-            gTextBoxHeight = MAXTEXTBOXHEIGHT;
             gMovingRect.size.height = [gBOINC_Logo size].height + gTextBoxHeight;
         }
     }
@@ -831,7 +885,10 @@ void launchedGfxApp(char * appPath, pid_t thePID, int slot) {
     // Check for a new graphics app sending us data
     if (UseSharedOffscreenBuffer() && gfxAppStartTime) {
         if (mySharedGraphicsController) {
-            [mySharedGraphicsController testConnection];
+            if (!runningSharedGraphics) {
+                // wait for a connection from a gfx app
+                [mySharedGraphicsController testConnection];
+            }
         }
     }
 }
@@ -1067,12 +1124,14 @@ static bool okToDraw;
     [[NSNotificationCenter defaultCenter] removeObserver:self
         name:NSPortDidBecomeInvalidNotification object:nil];
 
-    [[NSNotificationCenter defaultCenter] addObserver:self
-        selector:@selector(portDied:) name:NSPortDidBecomeInvalidNotification object:nil];
-
     openGLView = nil;
 
     [self testConnection];
+
+    if (! gMach_bootstrap_unavailable_to_screensavers)  {
+        [[NSNotificationCenter defaultCenter] addObserver:self
+            selector:@selector(portDied:) name:NSPortDidBecomeInvalidNotification object:nil];
+    }
 }
 
 
@@ -1080,92 +1139,147 @@ static bool okToDraw;
 {
     mach_port_t servicePortNum = MACH_PORT_NULL;
     kern_return_t machErr;
-    char *portName = "edu.berkeley.boincsaver";
+    char *portNameV1 = "edu.berkeley.boincsaver";
+    char *portNameV2 = "edu.berkeley.boincsaver-v2";
 
-	// Try to check in with master.
+    if ((!gMach_bootstrap_unavailable_to_screensavers) || (serverPort == MACH_PORT_NULL)) {
+       // Try to check in with master.
 // NSMachBootstrapServer is deprecated in OS 10.13, so use bootstrap_look_up
 //	serverPort = [(NSMachPort *)([[NSMachBootstrapServer sharedInstance] portForName:@"edu.berkeley.boincsaver"]) retain];
-	machErr = bootstrap_look_up(bootstrap_port, portName, &servicePortNum);
-    if (machErr == KERN_SUCCESS) {
-        serverPort = (NSMachPort*)[NSMachPort portWithMachPort:servicePortNum];
-    } else {
-        serverPort = MACH_PORT_NULL;
+        machErr = bootstrap_look_up(bootstrap_port, portNameV1, &servicePortNum);
+        if (machErr != KERN_SUCCESS) {
+            if (machErr != BOOTSTRAP_NOT_PRIVILEGED) {
+            //  bootstrap_look_up() returned error other than BOOTSTRAP_NOT_PRIVILEGED
+            // machErr is probably BOOTSTRAP_UNKNOWN_SERVICE because no gfx app is running
+            // Keep trying bootstrap_look_up() until we get a connection from gfx app
+            serverPort = MACH_PORT_NULL;
+            } else {    //  bootstrap_look_up() returned error BOOTSTRAP_NOT_PRIVILEGED
+                // As of MacOS 14.0, the legacyScreenSave sandbox prevents using
+                // bootstrap_look_up. I have filed bug report FB13300491 with
+                // Apple and hope they will change this in a future MacOS.
+                machErr = bootstrap_check_in(bootstrap_port, portNameV2, &servicePortNum);
+                if (machErr != KERN_SUCCESS) {
+                    [NSApp terminate:nil];
+                }
+                gMach_bootstrap_unavailable_to_screensavers = true;
+            }   //  bootstrap_look_up() returned error BOOTSTRAP_NOT_PRIVILEGED
+        }   // bootstrap_look_up() did not return KERN_SUCCESS
+
+         if (machErr == KERN_SUCCESS) {
+           serverPort = (NSMachPort*)[NSMachPort portWithMachPort:servicePortNum];
+        }
+
+        if ((serverPort != MACH_PORT_NULL) && (localPort == MACH_PORT_NULL)) {
+            // Retrieve raw mach port names.
+            serverPortName = [serverPort machPort];
+
+            if (gMach_bootstrap_unavailable_to_screensavers) {
+               // Register server port with the current runloop.
+                [serverPort setDelegate:self];
+                [serverPort scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
+            } else {    // (! gMach_bootstrap_unavailable_to_screensavers)
+                // Create our own local port.
+                localPort = [[NSMachPort alloc] init];
+                localPortName  = [localPort machPort];
+                // Register our local port with the current runloop.
+                [localPort setDelegate:self];
+                [localPort scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
+
+                // Check in with server.
+                int kr;
+                kr = _MGCCheckinClient(serverPortName, localPortName, &clientIndex);
+                if(kr != 0) {
+                    [NSApp terminate:nil];
+                }
+
+                runningSharedGraphics = true;
+
+                if (childPid) {
+                    gfxAppStartTime = 0.0;
+                    childApp = [NSRunningApplication runningApplicationWithProcessIdentifier:childPid];
+                }
+            }   // (! gMach_bootstrap_unavailable_to_screensavers)
+        }   // ((serverPort != MACH_PORT_NULL) && (localPort == MACH_PORT_NULL))
+    }   // ((!gMach_bootstrap_unavailable_to_screensavers) || (serverPort == MACH_PORT_NULL))
+}
+
+
+- (void)cleanUpOpenGL
+{
+    if (gMach_bootstrap_unavailable_to_screensavers) {
+        [serverPort removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
+    }
+    childPid=0;
+    childApp = nil;
+
+    if (gMach_bootstrap_unavailable_to_screensavers ||
+            ((serverPort == MACH_PORT_NULL) && (localPort == MACH_PORT_NULL))
+            ) {
+        if (openGLView) {
+            if (NSThread.isMainThread) {
+                [openGLView removeFromSuperview];   // Releases openGLView
+            } else {
+                dispatch_sync(dispatch_get_main_queue(), ^{
+                    [openGLView removeFromSuperview];   // Releases openGLView
+                });
+            }
+            openGLView = nil;
+        }
+
+    int i;
+    for(i = 0; i < NUM_IOSURFACE_BUFFERS; i++) {
+        if (_ioSurfaceBuffers[i]) {
+            CFRelease(_ioSurfaceBuffers[i]);
+            _ioSurfaceBuffers[i] = nil;
+        }
+
+        // if (glIsTexture(_textureNames[i])) {
+            // glDeleteTextures(1, _textureNames[i]);
+        // }
+        _textureNames[i] = 0;
+
+        if (_ioSurfaceMachPorts[i] != MACH_PORT_NULL) {
+            mach_port_deallocate(mach_task_self(), _ioSurfaceMachPorts[i]);
+            _ioSurfaceMachPorts[i] = MACH_PORT_NULL;
+        }
     }
 
-	if(serverPort != MACH_PORT_NULL)
-	{
-		// Create our own local port.
-		localPort = [[NSMachPort alloc] init];
-
-		// Retrieve raw mach port names.
-		serverPortName = [serverPort machPort];
-		localPortName  = [localPort machPort];
-
-		// Register our local port with the current runloop.
-		[localPort setDelegate:self];
-		[localPort scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
-
-		// Check in with server.
-		int kr;
-		kr = _MGCCheckinClient(serverPortName, localPortName, &clientIndex);
-		if(kr != 0)
-			[NSApp terminate:nil];
-
-        runningSharedGraphics = true;
-
-        if (childPid) {
-            gfxAppStartTime = 0.0;
-            childApp = [NSRunningApplication runningApplicationWithProcessIdentifier:childPid];
+        runningSharedGraphics = false;  // Do this last!!
+        if (gMach_bootstrap_unavailable_to_screensavers) {
+            [serverPort scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
         }
     }
 }
 
 - (void)portDied:(NSNotification *)notification
 {
+    if (gMach_bootstrap_unavailable_to_screensavers) {
+        return; // This should never be called if using MacOS 14 workaround
+    }
 	NSPort *port = [notification object];
 	if(port == serverPort) {
         childApp = nil;
         gfxAppStartTime = 0.0;
         gfxAppPath[0] = '\0';
+        gfxWuName[0] = '\0';
 
         if ([serverPort isValid]) {
             [serverPort invalidate];
 //            [serverPort release];
         }
-        serverPort = nil;
+        serverPort = MACH_PORT_NULL;
 		[localPort removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
 
         if ([localPort isValid]) {
             [localPort invalidate];
         }
 //        [localPort release];
-        localPort = nil;
+        localPort = MACH_PORT_NULL;
 
-        int i;
-        for(i = 0; i < NUM_IOSURFACE_BUFFERS; i++) {
-            if (_ioSurfaceBuffers[i]) {
-                CFRelease(_ioSurfaceBuffers[i]);
-                _ioSurfaceBuffers[i] = nil;
-            }
-
-            // if (glIsTexture(_textureNames[i])) {
-                // glDeleteTextures(1, _textureNames[i]);
-            // }
-            _textureNames[i] = 0;
-
-            if (_ioSurfaceMachPorts[i] != MACH_PORT_NULL) {
-                mach_port_deallocate(mach_task_self(), _ioSurfaceMachPorts[i]);
-                _ioSurfaceMachPorts[i] = MACH_PORT_NULL;
-            }
-        }
-
-        if ((serverPort == nil) && (localPort == nil)) {
-            runningSharedGraphics = false;
-            [openGLView removeFromSuperview];   // Releases openGLView
-            openGLView = nil;
-        }
+        [ self cleanUpOpenGL ];
 	}
 }
+
 - (void)handleMachMessage:(void *)msg
 {
 	union __ReplyUnion___MGCMGSServer_subsystem reply;
@@ -1177,13 +1291,16 @@ static bool okToDraw;
 	{
 		kr = mach_msg(reply_header, MACH_SEND_MSG, reply_header->msgh_size, 0, MACH_PORT_NULL,
 			     0, MACH_PORT_NULL);
-        if(kr != 0)
+        if(kr != 0) {
 			[NSApp terminate:nil];
+        }
 	}
 }
 
 - (kern_return_t)displayFrame:(int32_t)frameIndex surfacemachport:(mach_port_t)iosurface_port
 {
+    if (!childPid) return 0;
+
 	nextFrameIndex = frameIndex;
 
 	if(!_ioSurfaceBuffers[frameIndex])
@@ -1210,11 +1327,21 @@ static bool okToDraw;
 		_textureNames[frameIndex] = [openGLView setupIOSurfaceTexture:_ioSurfaceBuffers[frameIndex]];
     }
 
-    okToDraw = true;    // Tell drawRect that we have real data to display
+    if (openGLView) {
+        okToDraw = true;    // Tell drawRect that we have real data to display
 
-	[openGLView setNeedsDisplay:YES];
-	[openGLView display];
+        [openGLView setNeedsDisplay:YES];
+        [openGLView display];
+    }
 
+    if (gMach_bootstrap_unavailable_to_screensavers && (runningSharedGraphics == false)) {
+        // We have now heard from gfx app so connection has been established
+        runningSharedGraphics = true;
+        if (childPid) {
+            gfxAppStartTime = 0.0;
+            childApp = [NSRunningApplication runningApplicationWithProcessIdentifier:childPid];
+        }
+    }
 	return 0;
 }
 
@@ -1254,8 +1381,9 @@ kern_return_t _MGSDisplayFrame(mach_port_t server_port, int32_t frame_index, mac
 
     NSOpenGLPixelFormat *pix_fmt = [[NSOpenGLPixelFormat alloc] initWithAttributes:attribs];
 
-    if(!pix_fmt)
+    if(!pix_fmt) {
        [ NSApp terminate:nil];
+    }
 
 	self = [super initWithFrame:frame pixelFormat:pix_fmt];
 
