@@ -1,6 +1,6 @@
 // This file is part of BOINC.
-// http://boinc.berkeley.edu
-// Copyright (C) 2023 University of California
+// https://boinc.berkeley.edu
+// Copyright (C) 2024 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -15,6 +15,10 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
+// structures corresponding to various DB tables.
+// In some cases the structures have extra fields,
+// used by the server code but not stored in the DB
+
 #ifndef _BOINC_DB_TYPES_
 #define _BOINC_DB_TYPES_
 
@@ -372,8 +376,13 @@ struct HOST {
     bool p_vm_extensions_disabled;
     int num_opencl_cpu_platforms;
     OPENCL_CPU_PROP opencl_cpu_prop[MAX_OPENCL_CPU_PLATFORMS];
-    bool wsl_available;
-    WSLS wsls;
+    WSL_DISTROS wsl_distros;
+
+    // Docker info (non-Win only)
+    char docker_version[256];
+    int docker_type;
+    char docker_compose_version[256];
+    int docker_compose_type;
 
     // stuff from time_stats
     double cpu_and_network_available_frac;
