@@ -1,6 +1,6 @@
 # This file is part of BOINC.
-# http://boinc.berkeley.edu
-# Copyright (C) 2024 University of California
+# https://boinc.berkeley.edu
+# Copyright (C) 2025 University of California
 #
 # BOINC is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License
@@ -38,7 +38,7 @@ linux_apps_list = [
     './samples/example_app/ucn',
     './samples/example_app/uc2_graphics',
     './samples/example_app/slide_show',
-    './samples/multi_thread/multi_thread',
+    './samples/multi_thread/multi_thread*pc-linux-gnu',
     './samples/sleeper/sleeper',
     './samples/vboxmonitor/vboxmonitor',
     './samples/vboxwrapper/vboxwrapper*pc-linux-gnu',
@@ -47,7 +47,8 @@ linux_apps_list = [
     './samples/openclapp/openclapp',
     './samples/wrappture/wrappture_example',
     './samples/wrappture/fermi',
-    './samples/sporadic/sporadic'
+    './samples/sporadic/sporadic',
+    './samples/docker_wrapper/docker_wrapper*pc-linux-gnu',
 ]
 
 linux_manager_list = [
@@ -69,28 +70,15 @@ mingw_apps_vcpkg_list = [
     './samples/example_app/ucn.exe',
     './samples/example_app/uc2_graphics.exe',
     './samples/example_app/slide_show.exe',
-    './samples/multi_thread/multi_thread.exe',
+    './samples/multi_thread/multi_thread*.exe',
     './samples/sleeper/sleeper.exe',
     './samples/worker/worker*.exe',
     './samples/wrapper/wrapper*.exe',
     './samples/wrappture/wrappture_example.exe',
     './samples/wrappture/fermi.exe',
-    './samples/sporadic/sporadic.exe'
-]
-
-android_manager_generic_list = [
-    './android/BOINC/app/build/outputs/apk/debug/app-debug.apk',
-    './android/BOINC/app/build/outputs/apk/release/app-release-unsigned.apk'
-]
-
-android_manager_xiaomi_list = [
-    './android/BOINC/app/build/outputs/apk/xiaomi_debug/app-xiaomi_debug.apk',
-    './android/BOINC/app/build/outputs/apk/xiaomi_release/app-xiaomi_release-unsigned.apk'
-]
-
-android_manager_armv6_list = [
-    './android/BOINC/app/build/outputs/apk/armv6_debug/app-armv6_debug.apk',
-    './android/BOINC/app/build/outputs/apk/armv6_release/app-armv6_release-unsigned.apk'
+    './samples/sporadic/sporadic.exe',
+    './samples/wsl_wrapper/wsl_wrapper.exe',
+    './samples/docker_wrapper/docker_wrapper*.exe',
 ]
 
 android_apps_list = [
@@ -170,6 +158,8 @@ windows_apps_list = [
     './win_build/Build/x64/Release/test*.exe',
     './win_build/Build/x64/Release/wrappture*.exe',
     './win_build/Build/x64/Release/crypt_prog.exe',
+    './win_build/Build/x64/Release/wsl_wrapper.exe',
+    './win_build/Build/x64/Release/docker_wrapper*.exe',
     './win_build/Build/ARM64/Release/htmlgfx*.exe',
     './win_build/Build/ARM64/Release/wrapper*.exe',
     './win_build/Build/ARM64/Release/vboxwrapper*.exe',
@@ -182,7 +172,9 @@ windows_apps_list = [
     './win_build/Build/ARM64/Release/multi_thread*.exe',
     './win_build/Build/ARM64/Release/test*.exe',
     './win_build/Build/ARM64/Release/wrappture*.exe',
-    './win_build/Build/ARM64/Release/crypt_prog.exe'
+    './win_build/Build/ARM64/Release/crypt_prog.exe',
+    './win_build/Build/ARM64/Release/wsl_wrapper.exe',
+    './win_build/Build/ARM64/Release/docker_wrapper*.exe',
 ]
 
 windows_client_list = [
@@ -211,6 +203,14 @@ windows_manager_list = [
 windows_installer_list = [
     './win_build/Build/x64/Release/boinccas.dll',
     './win_build/Build/ARM64/Release/boinccas.dll',
+    './win_build/Build/x64/Release/installer_icon.exe',
+    './win_build/Build/ARM64/Release/installer_icon.exe',
+    './win_build/Build/x64/Release/installer.exe',
+    './win_build/Build/ARM64/Release/installer.exe',
+    './win_build/Build/x64/Release/boinc.msi',
+    './win_build/Build/ARM64/Release/boinc.msi',
+    './win_build/Build/x64/Release/installer_setup.exe',
+    './win_build/Build/ARM64/Release/installer_setup.exe',
 ]
 
 wasm_client_list = [
@@ -230,10 +230,6 @@ wasm_client_debug_folder_list = [
     'client/boinc_client.wasm',
 ]
 
-snap_list = [
-    './boinc_*.snap',
-]
-
 macos_manager_list = [
     'mac_build/build/Deployment/AddRemoveUser',
     'mac_build/build/Deployment/BOINC\ Installer.app',
@@ -257,6 +253,7 @@ macos_manager_list = [
 macos_apps_list = [
     'zip/build/Deployment/boinc_zip_test',
     'zip/build/Deployment/testzlibconflict',
+    'samples/docker_wrapper/build/Deployment/docker_wrapper',
     'samples/mac_build/build/Deployment/UC2-apple-darwin',
     'samples/mac_build/build/Deployment/UC2_graphics-apple-darwin',
     'samples/mac_build/build/Deployment/slide_show-apple-darwin',
@@ -269,6 +266,7 @@ macos_makefile_apps_list = [
 ]
 
 macos_apps_x86_64_list = [
+    'samples/docker_wrapper/docker_wrapper_x86_64',
     'samples/example_app/x86_64/uc2',
     'samples/example_app/uc2_x86_64',
     'samples/example_app/x86_64/uc2_graphics',
@@ -279,6 +277,7 @@ macos_apps_x86_64_list = [
 ]
 
 macos_apps_arm64_list = [
+    'samples/docker_wrapper/docker_wrapper_arm64',
     'samples/example_app/arm64/uc2',
     'samples/example_app/uc2_arm64',
     'samples/example_app/arm64/uc2_graphics',
@@ -299,6 +298,7 @@ logs_list = [
     'parts/boinc/build/3rdParty/linux/vcpkg/buildtrees/*.log',
     'android/BOINC/app/build/reports/',
     'mac_build/xcodebuild_*.log',
+    'build/*.log',
 ]
 
 def prepare_7z_archive(archive_name, target_directory, files_list):
@@ -323,6 +323,9 @@ def prepare_linux_apps(target_directory):
 def prepare_linux_apps_arm64(target_directory):
     prepare_7z_archive('linux_apps-arm64', target_directory, linux_apps_list)
 
+def prepare_linux_apps_vcpkg_arm64(target_directory):
+    prepare_7z_archive('linux_apps-vcpkg-arm64', target_directory, linux_apps_list)
+
 def prepare_linux_apps_vcpkg(target_directory):
     prepare_7z_archive('linux_apps-vcpkg', target_directory, linux_apps_list)
 
@@ -344,22 +347,9 @@ def prepare_win_apps_mingw(target_directory):
 def prepare_win_apps_mingw_vcpkg(target_directory):
     prepare_7z_archive('win_apps-mingw-vcpkg', target_directory, mingw_apps_vcpkg_list)
 
-def prepare_android_manager(target_directory):
-    prepare_7z_archive('android_manager', target_directory, android_manager_generic_list)
-    prepare_7z_archive('android_manager_xiaomi', target_directory, android_manager_xiaomi_list)
-    prepare_7z_archive('android_manager_armv6', target_directory, android_manager_armv6_list)
-
-def prepare_android_manager_vcpkg(target_directory):
-    prepare_7z_archive('android_manager-vcpkg', target_directory, android_manager_generic_list)
-    prepare_7z_archive('android_manager-vcpkg_xiaomi', target_directory, android_manager_xiaomi_list)
-    prepare_7z_archive('android_manager-vcpkg_armv6', target_directory, android_manager_armv6_list)
-
 def prepare_android_apps(target_directory):
     prepare_7z_archive('android_apps', target_directory, android_apps_list)
 
-def prepare_android_apps_vcpkg(target_directory):
-    prepare_7z_archive('android_apps-vcpkg', target_directory, android_apps_list)
-
 def prepare_win_apps(target_directory):
     prepare_7z_archive('win_apps', target_directory, windows_apps_list)
 
@@ -378,9 +368,6 @@ def prepare_wasm_client(target_directory):
 def prepare_wasm_client_debug(target_directory):
     prepare_7z_archive('wasm_client-debug', target_directory, wasm_client_debug_folder_list)
 
-def prepare_linux_snap(target_directory):
-    prepare_7z_archive('linux_snap', target_directory, snap_list)
-
 def prepare_macos_apps(target_directory):
     prepare_7z_archive('macos_manager', target_directory, macos_manager_list)
     prepare_7z_archive('macos_apps', target_directory, macos_apps_list)
@@ -398,6 +385,7 @@ boinc_types = {
     'linux_client-vcpkg': prepare_linux_client_vcpkg,
     'linux_apps': prepare_linux_apps,
     'linux_apps-arm64': prepare_linux_apps_arm64,
+    'linux_apps-vcpkg-arm64': prepare_linux_apps_vcpkg_arm64,
     'linux_apps-vcpkg': prepare_linux_apps_vcpkg,
     'linux_manager': prepare_linux_manager,
     'linux_manager-with-webview': prepare_linux_manager_with_webview,
@@ -405,17 +393,13 @@ boinc_types = {
     'linux_manager-without-webview': prepare_linux_manager_without_webview,
     'win_apps-mingw': prepare_win_apps_mingw,
     'win_apps-mingw-vcpkg': prepare_win_apps_mingw_vcpkg,
-    'android_manager': prepare_android_manager,
-    'android_manager-vcpkg': prepare_android_manager_vcpkg,
     'android_apps': prepare_android_apps,
-    'android_apps-vcpkg': prepare_android_apps_vcpkg,
     'win_apps': prepare_win_apps,
     'win_client': prepare_win_client,
     'win_manager': prepare_win_manager,
     'win_installer': prepare_win_installer,
     'wasm_client': prepare_wasm_client,
     'wasm_client-debug': prepare_wasm_client_debug,
-    'linux_snap': prepare_linux_snap,
     'macos_manager': prepare_macos_apps,
     'macos_samples-makefile': prepare_macos_makefile_apps,
     'logs': prepare_logs,
