@@ -36,6 +36,7 @@ require_once("../inc/util.inc");
 require_once("../inc/xml.inc");
 require_once("../inc/boinc_db.inc");
 require_once("../inc/server_version.inc");
+require_once("../inc/common_defs.inc");
 
 if (!defined('STATUS_PAGE_TTL')) {
     define('STATUS_PAGE_TTL', 3600);
@@ -256,10 +257,12 @@ function show_status_xml($x) {
         item_xml("name", $app->name);
         item_xml("unsent", $app->unsent);
         item_xml("in_progress", $app->in_progress);
-        item_xml("avg_runtime", $app->info->avg);
-        item_xml("min_runtime", $app->info->min);
-        item_xml("max_runtime", $app->info->max);
-        item_xml("users", $app->info->users);
+        if ($app->info) {
+            item_xml("avg_runtime", $app->info->avg);
+            item_xml("min_runtime", $app->info->min);
+            item_xml("max_runtime", $app->info->max);
+            item_xml("users", $app->info->users);
+        }
         echo "</app>\n";
     }
     echo "</tasks_by_app>
@@ -354,17 +357,13 @@ function get_daemon_status() {
         $have_remote = true;
     }
 
-    // Scheduler is a daemon too
-    //
-    if ($sched_host == $main_host) {
-        $y = new StdClass;
-        $y->cmd = "Scheduler";
-        $y->host = $sched_host;
-        $y->status = !file_exists("../../stop_sched");;
-        $local_daemons[] = $y;
-    } else {
-        $have_remote = true;
-    }
+    // the scheduler is a CGI program, not a daemon;
+    // it doesn't have a PID.
+    $y = new StdClass;
+    $y->cmd = "Scheduler";
+    $y->host = $sched_host;
+    $y->status = !file_exists("../../stop_sched");;
+    $local_daemons[] = $y;
 
     foreach ($daemons->daemon as $d) {
         if ((int)$d->disabled != 0) {
@@ -418,13 +417,17 @@ function get_job_status() {
     foreach ($apps as $app) {
         $info = BoincDB::get()->lookup_fields("result", "stdClass",
             "ceil(avg(elapsed_time)/3600*100)/100 as avg,
-            ceil(min(elapsed_time)/3600*100)/100 as min,
-            ceil(max(elapsed_time)/3600*100)/100 as max,
-            count(distinct userid) as users",
-            "appid = $app->id
-            AND validate_state=1
-            AND received_time > (unix_timestamp()-86400)
-            "
+                ceil(min(elapsed_time)/3600*100)/100 as min,
+                ceil(max(elapsed_time)/3600*100)/100 as max,
+                count(distinct userid) as users
+            ",
+            sprintf('appid=%d
+                AND validate_state=%d
+                AND received_time > (unix_timestamp()-86400)
+                ',
+                $app->id,
+                VALIDATE_STATE_VALID
+            )
         );
         // $info fields will be null if app has no results
         if ($info->avg) {
@@ -432,16 +435,34 @@ function get_job_status() {
         } else {
             $app->info = null;
         }
-        $app->unsent = BoincResult::count("appid=$app->id and server_state=2");
-        $app->in_progress = BoincResult::count("appid=$app->id and server_state=4");
+        $app->unsent = BoincResult::count(
+            sprintf('appid=%d and server_state=%d',
+                $app->id, RESULT_SERVER_STATE_UNSENT
+            )
+        );
+        $app->in_progress = BoincResult::count(
+            sprintf('appid=%d and server_state=%d',
+                $app->id, RESULT_SERVER_STATE_IN_PROGRESS
+            )
+        );
     }
     $s->apps = $apps;
-    $s->results_ready_to_send = BoincResult::count("server_state=2");
-    $s->results_in_progress = BoincResult::count("server_state=4");
-    $s->results_need_file_delete = BoincResult::count("file_delete_state=1");
+    $s->results_ready_to_send = BoincResult::count(
+        sprintf('server_state=%d', RESULT_SERVER_STATE_UNSENT)
+    );
+    $s->results_in_progress = BoincResult::count(
+        sprintf('server_state=%d', RESULT_SERVER_STATE_IN_PROGRESS)
+    );
+    $s->results_need_file_delete = BoincResult::count(
+        sprintf('file_delete_state=%d', FILE_DELETE_READY)
+    );
     $s->wus_need_validate = BoincWorkunit::count("need_validate=1");
-    $s->wus_need_assimilate = BoincWorkunit::count("assimilate_state=1");
-    $s->wus_need_file_delete = BoincWorkunit::count("file_delete_state=1");
+    $s->wus_need_assimilate = BoincWorkunit::count(
+        sprintf('assimilate_state=%d', ASSIMILATE_READY)
+    );
+    $s->wus_need_file_delete = BoincWorkunit::count(
+        sprintf('file_delete_state=%d', FILE_DELETE_READY)
+    );
     $x = BoincDB::get()->lookup_fields("workunit", "stdClass", "MIN(transition_time) as min", "TRUE");
     $gap = (time() - $x->min)/3600;
     if (($gap < 0) || ($x->min == 0)) {
@@ -470,12 +491,22 @@ function get_job_status() {
 function show_counts_xml() {
     xml_header();
     echo "<job_counts>\n";
-    item_xml('results_ready_to_send', BoincResult::count("server_state=2"));
-    item_xml('results_in_progress', BoincResult::count("server_state=4"));
-    item_xml('results_need_file_delete', BoincResult::count("file_delete_state=1"));
+    item_xml('results_ready_to_send', BoincResult::count(
+        sprintf('server_state=%d', RESULT_SERVER_STATE_UNSENT)
+    ));
+    item_xml('results_in_progress', BoincResult::count(
+        sprintf('server_state=%d', RESULT_SERVER_STATE_IN_PROGRESS)
+    ));
+    item_xml('results_need_file_delete', BoincResult::count(
+        sprintf('file_delete_state=%d', FILE_DELETE_READY)
+    ));
     item_xml('wus_need_validate', BoincWorkunit::count("need_validate=1"));
-    item_xml('wus_need_assimilate', BoincWorkunit::count("assimilate_state=1"));
-    item_xml('wus_need_file_delete', BoincWorkunit::count("file_delete_state=1"));
+    item_xml('wus_need_assimilate', BoincWorkunit::count(
+        sprintf('assimilate_state=%d', ASSIMILATE_READY)
+    ));
+    item_xml('wus_need_file_delete', BoincWorkunit::count(
+        sprintf('file_delete_state=%d', FILE_DELETE_READY)
+    ));
     echo "</job_counts>\n";
 }
 
