@@ -24,6 +24,7 @@ require_once("../inc/submit_db.inc");
 require_once("../inc/xml.inc");
 require_once("../inc/dir_hier.inc");
 require_once("../inc/result.inc");
+require_once("../inc/sandbox.inc");
 require_once("../inc/submit_util.inc");
 
 ini_set("memory_limit", "4G");
@@ -33,7 +34,7 @@ function get_wu($name) {
     $wu = BoincWorkunit::lookup("name='$name'");
     if (!$wu) {
         log_write("no job named $name was found");
-        xml_error(-1, "no job named $name was found");
+        xml_error(-1, "job not found: ".htmlspecialchars($name));
     }
     return $wu;
 }
@@ -43,7 +44,7 @@ function get_submit_app($name) {
     $app = BoincApp::lookup("name='$name'");
     if (!$app) {
         log_write("no app named $name was found");
-        xml_error(-1, "no app named $name was found");
+        xml_error(-1, "app not found: ".htmlspecialchars($name));
     }
     return $app;
 }
@@ -102,7 +103,7 @@ function read_input_template($app, $r) {
         $x = simplexml_load_file($path);
         if (!$x) {
             log_write("couldn't parse input template file $path");
-            xml_error(-1, "couldn't parse input template file $path");
+            xml_error(-1, "couldn't parse input template file ".htmlspecialchars($path));
         }
         return $x;
     } else {
@@ -110,13 +111,16 @@ function read_input_template($app, $r) {
     }
 }
 
-function check_max_jobs_in_progress($r, $user_submit) {
-    if (!$user_submit->max_jobs_in_progress) return;
+// if this batch would exceed user job limit, error out
+//
+function check_max_jobs_in_progress($r, $user) {
+    $us = BoincUserSubmit::lookup_userid($user->id);
+    if (!$us->max_jobs_in_progress) return;
     $query = "select count(*) as total from DBNAME.result, DBNAME.batch where batch.user_id=$userid and result.batch = batch.id and result.server_state<".RESULT_SERVER_STATE_OVER;
     $db = BoincDb::get();
     $n = $db->get_int($query, 'total');
     if ($n === false) return;
-    if ($n + count($r->batch->job) > $user_submit->max_jobs_in_progress) {
+    if ($n + count($r->batch->job) > $us->max_jobs_in_progress) {
         log_write("limit on jobs in progress exceeded");
         xml_error(-1, "limit on jobs in progress exceeded");
     }
@@ -125,7 +129,7 @@ function check_max_jobs_in_progress($r, $user_submit) {
 function estimate_batch($r) {
     xml_start_tag("estimate_batch");
     $app = get_submit_app((string)($r->batch->app_name));
-    list($user, $user_submit) = check_remote_submit_permissions($r, $app);
+    $user = check_remote_submit_permissions($r, $app);
 
     $template = read_input_template($app, $r);
     $e = est_elapsed_time($r, $template);
@@ -155,7 +159,7 @@ $fanout = parse_config(get_config(), "<uldl_dir_fanout>");
 
 // stage a file, and return the physical name
 //
-function stage_file($file) {
+function stage_file($file, $user) {
     global $fanout;
     $download_dir = parse_config(get_config(), "<download_dir>");
 
@@ -180,6 +184,14 @@ function stage_file($file) {
         return $name;
     case "local_staged":
         return $file->source;
+    case 'sandbox':
+        [$md5, $size] = sandbox_parse_info_file($user, $file->source);
+        if (!$md5) {
+            xml_error(-1, "sandbox link file $file->source not found");
+        }
+        $phys_name = job_file_name($md5);
+        $path = sandbox_path($user, $file->source);
+        stage_file_aux($path, $md5, $size, $phys_name);
     case "inline":
         $md5 = md5($file->source);
         if (!$md5) {
@@ -201,11 +213,11 @@ function stage_file($file) {
 
 // stage all the files
 //
-function stage_files(&$jobs) {
+function stage_files(&$jobs, $user) {
     foreach($jobs as $job) {
         foreach ($job->input_files as $file) {
             if ($file->mode != "remote") {
-                $file->name = stage_file($file);
+                $file->name = stage_file($file, $user);
             }
         }
     }
@@ -216,7 +228,8 @@ function stage_files(&$jobs) {
 function submit_jobs(
     $jobs, $job_params, $app, $batch_id, $priority, $app_version_num,
     $input_template_filename,        // batch-level; can also specify per job
-    $output_template_filename
+    $output_template_filename,
+    $user
 ) {
     global $input_templates, $output_templates;
 
@@ -259,12 +272,14 @@ function submit_jobs(
         $x .= "\n";
     }
 
-    $errfile = "/tmp/create_work_" . getmypid() . ".err";
     $cmd = "cd " . project_dir() . "; ./bin/create_work --appname $app->name --batch $batch_id";
+
+    if ($user->seti_id) {
+        $cmd .= " --target_user $user->id ";
+    }
     if ($priority !== null) {
         $cmd .= " --priority $priority";
     }
-
     if ($input_template_filename) {
         $cmd .= " --wu_template templates/$input_template_filename";
     }
@@ -289,7 +304,15 @@ function submit_jobs(
     if ($job_params->delay_bound) {
         $cmd .= " --delay_bound $job_params->delay_bound";
     }
-    $cmd .= " --stdin >$errfile 2>&1";
+    $cmd .= " --stdin ";
+
+    // send stdin/stderr to a temp file
+    $errfile = sprintf('/tmp/create_work_%d.err', getmypid());
+    $cmd .= sprintf(' >%s 2>&1', $errfile);
+
+    //echo "command: $cmd\n";
+    //echo "stdin: $x\n";
+
     $h = popen($cmd, "w");
     if ($h === false) {
         xml_error(-1, "can't run create_work");
@@ -372,7 +395,9 @@ function xml_get_jobs($r) {
         $job->input_template = null;
         if ($j->input_template) {
             $job->input_template = $j->input_template;
-            $job->input_template_xml = $j->input_template->asXML();
+            $x = $j->input_template->asXML();
+            $x = str_replace('<file_info/>', '<file_info></file_info>', $x);
+            $job->input_template_xml = $x;
         }
         $job->output_template = null;
         if ($j->output_template) {
@@ -436,18 +461,22 @@ function logical_end_time($r, $jobs, $user, $app) {
     return (double)$x;
 }
 
+function make_batch_name($user, $app) {
+    return sprintf('%s:%s:%s', $user->name, $app->name, date(DATE_RFC2822));
+}
+
 // $r is a simplexml object encoding the request message
 //
 function submit_batch($r) {
     xml_start_tag("submit_batch");
     $app = get_submit_app((string)($r->batch->app_name));
-    list($user, $user_submit) = check_remote_submit_permissions($r, $app);
+    $user = check_remote_submit_permissions($r, $app);
     $jobs = xml_get_jobs($r);
     $template = read_input_template($app, $r);
     if ($template) {
         validate_batch($jobs, $template);
     }
-    stage_files($jobs);
+    stage_files($jobs, $user);
     $njobs = count($jobs);
     $now = time();
     $app_version_num = (int)($r->batch->app_version_num);
@@ -492,6 +521,9 @@ function submit_batch($r) {
         log_write("adding jobs to existing batch $batch_id");
     } else {
         $batch_name = (string)($r->batch->batch_name);
+        if (!$batch_name) {
+            $batch_name = make_batch_name($user, $app);
+        }
         $batch_name = BoincDb::escape_string($batch_name);
         $state = BATCH_STATE_INIT;
         $batch_id = BoincBatch::insert(
@@ -520,7 +552,8 @@ function submit_batch($r) {
     submit_jobs(
         $jobs, $job_params, $app, $batch_id, $priority, $app_version_num,
         $input_template_filename,
-        $output_template_filename
+        $output_template_filename,
+        $user
     );
 
     // set state to IN_PROGRESS only after creating jobs;
@@ -543,9 +576,12 @@ function submit_batch($r) {
 function create_batch($r) {
     xml_start_tag("create_batch");
     $app = get_submit_app((string)($r->app_name));
-    list($user, $user_submit) = check_remote_submit_permissions($r, $app);
+    $user = check_remote_submit_permissions($r, $app);
     $now = time();
     $batch_name = (string)($r->batch_name);
+    if (!$batch_name) {
+        $batch_name = make_batch_name($user, $app);
+    }
     $batch_name = BoincDb::escape_string($batch_name);
     $expire_time = (double)($r->expire_time);
     $state = BATCH_STATE_INIT;
@@ -586,7 +622,7 @@ function print_batch_params($batch, $get_cpu_time) {
 
 function query_batches($r) {
     xml_start_tag("query_batches");
-    list($user, $user_submit) = check_remote_submit_permissions($r, null);
+    $user = check_remote_submit_permissions($r, null);
     $batches = BoincBatch::enum("user_id = $user->id");
     $get_cpu_time = (int)($r->get_cpu_time);
     foreach ($batches as $batch) {
@@ -688,7 +724,7 @@ function get_batch($r) {
 
 function query_batch($r) {
     xml_start_tag("query_batch");
-    list($user, $user_submit) = check_remote_submit_permissions($r, null);
+    $user = check_remote_submit_permissions($r, null);
     $batch = get_batch($r);
     if ($batch->user_id != $user->id) {
         log_write("not owner of batch");
@@ -730,7 +766,7 @@ function results_sent($wu) {
 //
 function query_batch2($r) {
     xml_start_tag("query_batch2");
-    list($user, $user_submit) = check_remote_submit_permissions($r, null);
+    $user = check_remote_submit_permissions($r, null);
     $batch_names = $r->batch_name;
     $batches = array();
     foreach ($batch_names as $b) {
@@ -789,7 +825,7 @@ function query_batch2($r) {
 
 function query_job($r) {
     xml_start_tag("query_job");
-    list($user, $user_submit) = check_remote_submit_permissions($r, null);
+    $user = check_remote_submit_permissions($r, null);
     $job_id = (int)($r->job_id);
     $wu = BoincWorkunit::lookup_id($job_id);
     if (!$wu) {
@@ -832,7 +868,7 @@ function query_job($r) {
 //
 function query_completed_job($r) {
     xml_start_tag("query_completed_job");
-    list($user, $user_submit) = check_remote_submit_permissions($r, null);
+    $user = check_remote_submit_permissions($r, null);
     $job_name = (string)($r->job_name);
     $job_name = BoincDb::escape_string($job_name);
     $wu = BoincWorkunit::lookup("name='$job_name'");
@@ -881,7 +917,7 @@ function query_completed_job($r) {
 
 function handle_abort_batch($r) {
     xml_start_tag("abort_batch");
-    list($user, $user_submit) = check_remote_submit_permissions($r, null);
+    $user = check_remote_submit_permissions($r, null);
     $batch = get_batch($r);
     if ($batch->user_id != $user->id) {
         log_write("not owner");
@@ -897,7 +933,7 @@ function handle_abort_batch($r) {
 //
 function handle_abort_jobs($r) {
     xml_start_tag("abort_jobs");
-    list($user, $user_submit) = check_remote_submit_permissions($r, null);
+    $user = check_remote_submit_permissions($r, null);
     $batch = null;
     foreach ($r->job_name as $job_name) {
         $job_name = BoincDb::escape_string($job_name);
@@ -927,7 +963,7 @@ function handle_abort_jobs($r) {
 
 function handle_retire_batch($r) {
     xml_start_tag("retire_batch");
-    list($user, $user_submit) = check_remote_submit_permissions($r, null);
+    $user = check_remote_submit_permissions($r, null);
     $batch = get_batch($r);
     if ($batch->user_id != $user->id) {
         log_write("not owner of batch");
@@ -941,7 +977,7 @@ function handle_retire_batch($r) {
 
 function handle_set_expire_time($r) {
     xml_start_tag("set_expire_time");
-    list($user, $user_submit) = check_remote_submit_permissions($r, null);
+    $user = check_remote_submit_permissions($r, null);
     $batch = get_batch($r);
     if ($batch->user_id != $user->id) {
         log_write("not owner of batch");
@@ -968,7 +1004,7 @@ function get_templates($r) {
         $app = BoincApp::lookup_id($wu->appid);
     }
 
-    list($user, $user_submit) = check_remote_submit_permissions($r, $app);
+    $user = check_remote_submit_permissions($r, $app);
     $in = file_get_contents(project_dir() . "/templates/".$app->name."_in");
     $out = file_get_contents(project_dir() . "/templates/".$app->name."_out");
     if ($in === false || $out === false) {
@@ -1044,6 +1080,13 @@ estimate_batch($r);
 exit;
 }
 
+xml_header();
+if (0) {
+    $req = file_get_contents("req");
+} else {
+    $req = $_POST['request'];
+}
+
 // optionally write request message (XML) to log file
 //
 $request_log = parse_config(get_config(), "<remote_submit_request_log>");
@@ -1051,21 +1094,15 @@ if ($request_log) {
     $log_dir = parse_config(get_config(), "<log_dir>");
     $request_log = $log_dir . "/" . $request_log;
     if ($file = fopen($request_log, "a")) {
-        fwrite($file, "\n<submit_rpc_handler date=\"" . date(DATE_ATOM) . "\">\n" . $_POST['request'] . "\n</submit_rpc_handler>\n");
+        fwrite($file, "\n<submit_rpc_handler date=\"" . date(DATE_ATOM) . "\">\n" . $req . "\n</submit_rpc_handler>\n");
         fclose($file);
     }
 }
 
-xml_header();
-if (0) {
-    $req = file_get_contents("submit_req.xml");
-} else {
-    $req = $_POST['request'];
-}
 $r = simplexml_load_string($req);
 if (!$r) {
     log_write("----- RPC request: can't parse request message: $req");
-    xml_error(-1, "can't parse request message: $req");
+    xml_error(-1, "can't parse request message: ".htmlspecialchars($req));
 }
 
 log_write("----- Handling RPC; command ".$r->getName());
@@ -1087,7 +1124,7 @@ switch ($r->getName()) {
     case 'submit_batch': submit_batch($r); break;
     default:
         log_write("bad command");
-        xml_error(-1, "bad command: ".$r->getName());
+        xml_error(-1, "bad command");
         break;
 }
 
