@@ -80,7 +80,12 @@ function show_admins($user, $teamid) {
     start_table();
     row1(tra("Add Team Admin"));
     row2(tra("Email address of team member:"), '<input class="form-control" name="email_addr">');
-    row2("", "<input class=\"btn btn-success\" type=submit action value=\"".tra("Add")."\">");
+    row2("",
+        sprintf('<input class="btn" %s type=submit action value="%s">',
+            button_style(),
+            tra("Add")
+        )
+    );
     end_table();
     echo "</form>";
 
