@@ -1,6 +1,6 @@
 // This file is part of BOINC.
-// http://boinc.berkeley.edu
-// Copyright (C) 2023 University of California
+// https://boinc.berkeley.edu
+// Copyright (C) 2025 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -24,6 +24,7 @@
 #include "config.h"
 #include <cstdio>
 #include <cstring>
+#include <string.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -37,6 +38,8 @@
 
 #include "hostinfo.h"
 
+using std::string;
+
 HOST_INFO::HOST_INFO() {
     clear_host_info();
 }
@@ -71,9 +74,13 @@ void HOST_INFO::clear_host_info() {
     safe_strcpy(os_name, "");
     safe_strcpy(os_version, "");
 
-    wsl_available = false;
-#ifdef _WIN64
-    wsls.clear();
+#ifdef _WIN32
+    wsl_distros.clear();
+#else
+    safe_strcpy(docker_version, "");
+    docker_type = NONE;
+    safe_strcpy(docker_compose_version, "");
+    docker_compose_type = NONE;
 #endif
 
     safe_strcpy(product_name, "");
@@ -132,10 +139,21 @@ int HOST_INFO::parse(XML_PARSER& xp, bool static_items_only) {
         if (xp.parse_double("d_free", d_free)) continue;
         if (xp.parse_str("os_name", os_name, sizeof(os_name))) continue;
         if (xp.parse_str("os_version", os_version, sizeof(os_version))) continue;
-#ifdef _WIN64
-        if (xp.parse_bool("os_wsl_enabled", wsl_available)) continue;
+#ifdef _WIN32
         if (xp.match_tag("wsl")) {
-            this->wsls.parse(xp);
+            this->wsl_distros.parse(xp);
+            continue;
+        }
+#else
+        int i;
+        if (xp.parse_str("docker_version", docker_version, sizeof(docker_version))) continue;
+        if (xp.parse_int("docker_type", i)) {
+            docker_type = (DOCKER_TYPE)i;
+            continue;
+        }
+        if (xp.parse_str("docker_compose_version", docker_compose_version, sizeof(docker_compose_version))) continue;
+        if (xp.parse_int("docker_compose_type", i)) {
+            docker_compose_type = (DOCKER_TYPE)i;
             continue;
         }
 #endif
@@ -206,8 +224,7 @@ int HOST_INFO::write(
         "    <d_free>%f</d_free>\n"
         "    <os_name>%s</os_name>\n"
         "    <os_version>%s</os_version>\n"
-        "    <n_usable_coprocs>%d</n_usable_coprocs>\n"
-        "    <wsl_available>%d</wsl_available>\n",
+        "    <n_usable_coprocs>%d</n_usable_coprocs>\n",
         host_cpid,
         p_ncpus,
         pv,
@@ -225,16 +242,26 @@ int HOST_INFO::write(
         d_free,
         osn,
         osv,
-        coprocs.ndevs(),
-#ifdef _WIN64
-        wsl_available ? 1 : 0
-#else
-        0
-#endif
+        coprocs.ndevs()
     );
-#ifdef _WIN64
-    if (wsl_available) {
-        wsls.write_xml(out);
+#ifdef _WIN32
+    wsl_distros.write_xml(out);
+#else
+    if (strlen(docker_version)) {
+        out.printf(
+            "    <docker_version>%s</docker_version>\n"
+            "    <docker_type>%d</docker_type>\n",
+            docker_version,
+            docker_type
+        );
+    }
+    if (strlen(docker_compose_version)) {
+        out.printf(
+            "    <docker_compose_version>%s</docker_compose_version>\n"
+            "    <docker_compose_type>%d</docker_compose_type>\n",
+            docker_compose_version,
+            docker_compose_type
+        );
     }
 #endif
     if (strlen(product_name)) {
@@ -317,3 +344,114 @@ int HOST_INFO::write_cpu_benchmarks(FILE* out) {
     );
     return 0;
 }
+
+// name of CLI program
+//
+const char* docker_cli_prog(DOCKER_TYPE type) {
+    switch (type) {
+    case DOCKER: return "docker";
+#ifdef __APPLE__
+        case PODMAN: return "/opt/podman/bin/podman";
+#else
+    case PODMAN: return "podman";
+#endif
+    default: break;
+    }
+    return "unknown";
+}
+
+// display name
+//
+const char* docker_type_str(DOCKER_TYPE type) {
+    switch (type) {
+    case DOCKER: return "Docker";
+    case PODMAN: return "podman";
+    default: break;
+    }
+    return "unknown";
+}
+
+// on Mac+podman we need to set env variables
+// to use a directory accessable to boinc_master and boinc_projects
+//
+#ifdef __APPLE__
+const char* docker_cmd_prefix(DOCKER_TYPE type) {
+    static char buf[256];
+    if (type == PODMAN) {
+        const char* dir = "/Library/Application\ Support/BOINC\ Data/podman";
+        // must end w/ space
+        snprintf(buf, sizeof(buf),
+            "env XDG_CONFIG_HOME=\"%s\" XDG_DATA_HOME=\"%s\" ",
+            dir, dir
+        );
+        return buf;
+    }
+    return "";
+}
+#else
+const char* docker_cmd_prefix(DOCKER_TYPE) {
+    return "";
+}
+#endif
+
+// parse a string like
+// Docker version 24.0.7, build 24.0.7-0ubuntu2~22.04.1
+// or
+// podman version 4.9.3
+// ... possibly with a \n at the end.
+// Return the version (24.0.7 or 4.9.3)
+//
+bool HOST_INFO::get_docker_version_string(
+    DOCKER_TYPE /*type*/, const char* raw, string &version
+) {
+    char *p, *q;
+    const char *prefix;
+    prefix = "version ";
+    p = (char*)strstr(raw, prefix);
+    if (!p) return false;
+    p += strlen(prefix);
+    q = (char*)strstr(p, ",");
+    if (!q) {
+        q = (char*)strstr(p, " ");
+    }
+    if (!q) {
+        q = (char*)strstr(p, "\n");
+    }
+    if (!q) return false;
+    *q = 0;
+    version = p;
+    return true;
+}
+
+bool HOST_INFO::get_docker_compose_version_string(
+    DOCKER_TYPE type, const char *raw, string& version
+) {
+    char *p, *q;
+    const char* prefix;
+    switch (type) {
+    case DOCKER:
+        // Docker Compose version v2.17.3
+        prefix = "Docker Compose version v";
+        p = (char*)strstr(raw, prefix);
+        if (!p) return false;
+        p += strlen(prefix);
+        q = (char*)strstr(p, "\n");
+        if (q) *q = 0;
+        version = p;
+        return true;
+    // not sure about podman case
+    default: break;
+    }
+    return false;
+}
+
+bool HOST_INFO::have_docker() {
+#ifdef _WIN32
+    for (WSL_DISTRO &wd: wsl_distros.distros) {
+        if (!empty(wd.docker_version)) return true;
+    }
+    return false;
+#else
+    return strlen(docker_version)>0;
+#endif
+}
