@@ -123,9 +123,11 @@ void find_children(PROC_MAP& pm) {
     }
 }
 
-// get resource usage of non-BOINC apps
+// get resource usage of non-BOINC apps running above background priority.
 // NOTE: this is flawed because it doesn't account for short-lived processes.
-// It's not used on Win, Mac, or Linux, which have better ways of getting total CPU usage.
+// It's not used on Win, Mac, or Linux,
+// which have ways of getting total CPU usage.
+// See client/app.cpp
 //
 void procinfo_non_boinc(PROCINFO& procinfo, PROC_MAP& pm) {
     procinfo.clear();
@@ -162,10 +164,22 @@ void procinfo_non_boinc(PROCINFO& procinfo, PROC_MAP& pm) {
 #endif
 }
 
-// get CPU time of BOINC-related processes, low-priority processes,
-// and (if we're using Vbox) the Vbox daemon.
+// get CPU time of things we don't want to count as non-BOINC-related
+// - BOINC apps
+// - low-priority processes
+// - (if Vbox apps are running) the Vbox daemon
+// - Windows: WSL daemon ('vmmem')
+// - Linux/Mac:
+//      we don't account Docker/podman CPU time here,
+//      since we don't know what the processes are.
+//      Instead we do it in the client (by looking at ACTIVE_TASKS)
 //
-double boinc_related_cpu_time(PROC_MAP& pm, bool using_vbox) {
+//      processes named 'podman'
+//
+// This is subtracted from total CPU time to get
+// the 'non-BOINC CPU time' used in computing preferences
+//
+double boinc_related_cpu_time(PROC_MAP& pm, bool vbox_app_running) {
     double sum = 0;
     PROC_MAP::iterator i;
     for (i=pm.begin(); i!=pm.end(); ++i) {
@@ -176,9 +190,15 @@ double boinc_related_cpu_time(PROC_MAP& pm, bool using_vbox) {
         if (
             p.is_boinc_app
             || p.is_low_priority
-            || (using_vbox && strstr(p.command, "VBoxSVC"))
+            || (vbox_app_running && strstr(p.command, "VBox"))
+                // if a VBox app is running,
+                // count VBox processes as BOINC-related
+                // e.g. VBoxHeadless.exe and VBoxSVC.exe on Win
+#ifdef _WIN32
+            || strstr(p.command, "vmmem")
+#endif
         ) {
-            sum += p.user_time;
+            sum += (p.user_time + p.kernel_time);
         }
     }
     return sum;
