@@ -93,7 +93,7 @@ int setenv(const char *name, const char *value, int overwrite) {
         errno=ENOMEM;
         return -1;
     }
-    snprintf(buf, sizeof(buf),"%s=%s",name,value);
+    sprintf(buf, "%s=%s",name,value);
     rv=putenv(buf);
     // Yes, there is a potential memory leak here.  Some versions of operating
     // systems copy the string into the environment, others make the
