@@ -84,6 +84,14 @@ struct VBOX_VM : VBOX_BASE {
         double bytes_sent,
         double bytes_received
     );
+#ifdef _WIN32
+    int set_race_mitigation_lock(HANDLE& fd_race_mitigator, string& lock_name, const string& medium_file);
+    void remove_race_mitigation_lock(HANDLE& fd_race_mitigator, string& lock_name);
+#else
+    int set_race_mitigation_lock(int& fd_race_mitigator, string& lock_name, const string& medium_file);
+    void remove_race_mitigation_lock(int& fd_race_mitigator, string& lock_name);
+#endif
+    int remove_vbox_disk_orphans(string vbox_disk);
 };
 
 #endif
