@@ -29,6 +29,9 @@
 // --use_tstp       use SIGTSTP instead of SIGSTOP to suspend children
 //                  (Unix only).  SIGTSTP can be caught.
 //                  Use this if the wrapped program is itself a wrapper.
+// --passthrough_child
+//                  cmdline args beyond this one are passed to
+//                  tasks for which <append_cmdline_args> is set in job.xml
 //
 // Handles:
 // - suspend/resume/quit/abort
@@ -94,6 +97,8 @@ using std::string;
 
 bool use_tstp = false;
 
+//#define DEBUG
+
 #ifdef DEBUG
 inline void debug_msg(const char* x) {
     fprintf(stderr, "[DEBUG] %s\n", x);
@@ -139,6 +144,7 @@ struct TASK {
         // contribution of this task to overall fraction done
     bool is_daemon;
     bool append_cmdline_args;
+        // append wrapper args following --passthrough_child
     bool multi_process;
     bool forward_slashes;
     double time_limit;
@@ -167,7 +173,7 @@ struct TASK {
     int parse(XML_PARSER&);
     void substitute_macros();
     bool poll(int& status);
-    int run(int argc, char** argv);
+    int run(const vector<string> &child_args);
     void kill();
     void stop();
     void resume();
@@ -196,7 +202,7 @@ struct TASK {
         double temp, frac = 0;
         while (!feof(f)) {
             char buf[256];
-            char* p = fgets(buf, 256, f);
+            char* p = fgets(buf, sizeof(buf), f);
             if (p == NULL) break;
             int n = sscanf(buf, "%lf", &temp);
             if (n == 1) frac = temp;
@@ -281,41 +287,38 @@ void str_replace_all(string &str, const string& s1, const string& s2) {
 //
 void macro_substitute(string &str) {
     const char* pd = strlen(aid.project_dir)?aid.project_dir:".";
-    str_replace_all(str, "$PROJECT_DIR", pd);
-#ifdef DEBUG
-    fprintf(stderr, "[DEBUG] replacing '%s' with '%s'\n", "$PROJECT_DIR", pd);
+    char nt[256], cwd[1024];
+
+    sprintf(nt, "%d", nthreads);
+#ifdef _WIN32
+    GetCurrentDirectory(sizeof(cwd), cwd);
+#else
+    getcwd(cwd, sizeof(cwd));
 #endif
 
-    char nt[256];
-    sprintf(nt, "%d", nthreads);
-    str_replace_all(str, "$NTHREADS", nt);
 #ifdef DEBUG
-    fprintf(stderr, "[DEBUG] replacing '%s' with '%s'\n", "$NTHREADS", nt);
+    fprintf(stderr, "[DEBUG] macro_substitute '%s'\n", str.c_str());
+    fprintf(stderr, "[DEBUG] replacing $PROJECT_DIR with '%s'\n", pd);
+    fprintf(stderr, "[DEBUG] replacing $NTHREADS with '%s'\n", nt);
+    fprintf(stderr, "[DEBUG] replacing $PWD with '%s'\n", cwd);
 #endif
+    str_replace_all(str, "$PROJECT_DIR", pd);
+    str_replace_all(str, "$NTHREADS", nt);
+    str_replace_all(str, "$PWD", cwd);
 
     if (aid.gpu_device_num >= 0) {
         gpu_device_num = aid.gpu_device_num;
     }
     if (gpu_device_num >= 0) {
-        sprintf(nt, "%d", gpu_device_num);
-        str_replace_all(str, "$GPU_DEVICE_NUM", nt);
+        char buf[256];
+        sprintf(buf, "%d", gpu_device_num);
+        str_replace_all(str, "$GPU_DEVICE_NUM", buf);
 #ifdef DEBUG
-	fprintf(stderr, "[DEBUG] replacing '%s' with '%s'\n", "$GPU_DEVICE_NUM", nt);
+        fprintf(stderr, "[DEBUG] replacing $GPU_DEVICE_NUM with '%s'\n", buf);
 #endif
     }
-
-#ifdef _WIN32
-    GetCurrentDirectory(sizeof(nt),nt);
-    str_replace_all(str, "$PWD", nt);
 #ifdef DEBUG
-    fprintf(stderr, "[DEBUG] replacing '%s' with '%s'\n", "$PWD", nt);
-#endif
-#else
-    char cwd[1024];
-    str_replace_all(str, "$PWD", getcwd(cwd, sizeof(cwd)));
-#ifdef DEBUG
-    fprintf(stderr, "[DEBUG] replacing '%s' with '%s'\n", "$PWD", getcwd(cwd, sizeof(cwd)));
-#endif
+    fprintf(stderr, "[DEBUG] macro_substitute result '%s'\n", str.c_str());
 #endif
 }
 
@@ -331,8 +334,8 @@ void get_initial_file_list() {
     }
     dir_close(d);
     FILE* f = fopen("initial_file_list_temp", "w");
-    for (unsigned int i=0; i<initial_files.size(); i++) {
-        fprintf(f, "%s\n", initial_files[i].c_str());
+    for (string &s: initial_files) {
+        fprintf(f, "%s\n", s.c_str());
     }
     fclose(f);
     int retval = boinc_rename("initial_file_list_temp", "initial_file_list");
@@ -375,8 +378,8 @@ void do_unzip_inputs() {
 }
 
 bool in_vector(string s, vector<string>& v) {
-    for (unsigned int i=0; i<v.size(); i++) {
-        if (s == v[i]) return true;
+    for (string &s2: v) {
+        if (s == s2) return true;
     }
     return false;
 }
@@ -392,9 +395,9 @@ void get_zip_inputs(ZipFileList &files) {
     while (!dir_scan(fname, d, sizeof(fname))) {
         string filename = string(fname);
         if (in_vector(filename, initial_files)) continue;
-        for (unsigned int i=0; i<zip_patterns.size(); i++) {
+        for (regexp* re: zip_patterns) {
             regmatch match;
-            if (re_exec_w(zip_patterns[i], fname, 1, &match) == 1) {
+            if (re_exec_w(re, fname, 1, &match) == 1) {
                 files.push_back(filename);
                 break;
             }
@@ -541,7 +544,7 @@ int parse_job_file() {
     MIOFILE mf;
     char buf[256], buf2[256];
 
-    boinc_resolve_filename(JOB_FILENAME, buf, 1024);
+    boinc_resolve_filename(JOB_FILENAME, buf, sizeof(buf));
     FILE* f = boinc_fopen(buf, "r");
     if (!f) {
         fprintf(stderr,
@@ -596,10 +599,9 @@ int parse_job_file() {
     return ERR_XML_PARSE;
 }
 
-int start_daemons(int argc, char** argv) {
-    for (unsigned int i=0; i<daemons.size(); i++) {
-        TASK& task = daemons[i];
-        int retval = task.run(argc, argv);
+int start_daemons(const vector<string>& child_args) {
+    for (TASK& task: daemons) {
+        int retval = task.run(child_args);
         if (retval) return retval;
     }
     return 0;
@@ -607,8 +609,7 @@ int start_daemons(int argc, char** argv) {
 
 void kill_daemons() {
     vector<int> daemon_pids;
-    for (unsigned int i=0; i<daemons.size(); i++) {
-        TASK& task = daemons[i];
+    for (TASK& task: daemons) {
         if (task.pid) {
             daemon_pids.push_back(task.pid);
         }
@@ -677,7 +678,7 @@ void backslash_to_slash(char* p) {
     }
 }
 
-int TASK::run(int argct, char** argvt) {
+int TASK::run(const vector<string> &child_args) {
     string stdout_path, stdin_path, stderr_path;
     char app_path[1024], buf[256];
 
@@ -699,13 +700,13 @@ int TASK::run(int argct, char** argvt) {
         exit(1);
     }
 
-    // Optionally append wrapper's command-line arguments
+    // Optionally append wrapper's pass-through args
     // to those in the job file.
     //
     if (append_cmdline_args) {
-        for (int i=1; i<argct; i++){
+        for (const string& arg: child_args) {
             command_line += string(" ");
-            command_line += argvt[i];
+            command_line += arg;
         }
     }
 
@@ -1142,6 +1143,31 @@ int read_checkpoint(int& ntasks_completed, double& cpu, double& rt) {
     return 0;
 }
 
+// if the given file is a soft link of the form ../../project_dir/x,
+// return x, else return empty string
+//
+string resolve_proj_soft_link(const char* project_dir, const char* file) {
+    char buf[1024], physical_name[1024];
+    FILE* fp = boinc_fopen(file, "r");
+    if (!fp) {
+        return string("");
+    }
+    buf[0] = 0;
+    char* p = fgets(buf, sizeof(buf), fp);
+    fclose(fp);
+    if (!p) {
+        return string("");
+    }
+    if (!parse_str(buf, "<soft_link>", physical_name, sizeof(physical_name))) {
+        return string("");
+    }
+    snprintf(buf, sizeof(buf), "../../%s/", project_dir);
+    if (strstr(physical_name, buf) != physical_name) {
+        return string("");
+    }
+    return string(physical_name + strlen(buf));
+}
+
 // Check whether executable files (tasks and daemons) are code-signed.
 // The client supplies a list of app version files, which are code-signed.
 // For each executable file:
@@ -1152,7 +1178,7 @@ int read_checkpoint(int& ntasks_completed, double& cpu, double& rt) {
 void check_execs(vector<TASK> &t) {
     for (unsigned int i=0; i<t.size(); i++) {
         TASK &task = t[i];
-        string phys_name = resolve_soft_link(
+        string phys_name = resolve_proj_soft_link(
             aid.project_dir, task.application.c_str()
         );
         if (phys_name.empty()) {
@@ -1185,6 +1211,7 @@ void usage() {
         "   --trickle X\n"
         "   --version\n"
         "   --use_tstp\n"
+        "   --passthrough_child\n"
     );
     boinc_finish(EXIT_INIT_FAILURE);
 }
@@ -1198,9 +1225,9 @@ int main(int argc, char** argv) {
         // total CPU time at last checkpoint
     char buf[256];
     bool is_sporadic = false;
+    bool passthrough_child = false;
+    vector<string> child_args;
 
-    // Log banner
-    //
     fprintf(stderr, "%s wrapper (%d.%d.%d): starting\n",
         boinc_msg_prefix(buf, sizeof(buf)),
         BOINC_MAJOR_VERSION, BOINC_MINOR_VERSION, WRAPPER_RELEASE
@@ -1211,7 +1238,9 @@ int main(int argc, char** argv) {
 #endif
 
     for (int j=1; j<argc; j++) {
-        if (!strcmp(argv[j], "--nthreads")) {
+        if (passthrough_child) {
+            child_args.push_back(argv[j]);
+        } else if (!strcmp(argv[j], "--nthreads")) {
             nthreads = atoi(argv[++j]);
         } else if (!strcmp(argv[j], "--device")) {
             gpu_device_num = atoi(argv[++j]);
@@ -1226,6 +1255,8 @@ int main(int argc, char** argv) {
 #endif
         } else if (!strcmp(argv[j], "--use_tstp")) {
             use_tstp = true;
+        } else if (!strcmp(argv[j], "--passthrough_child")) {
+            passthrough_child = true;
         } else {
             fprintf(stderr, "Unrecognized option %s\n", argv[j]);
             usage();
@@ -1241,16 +1272,18 @@ int main(int argc, char** argv) {
         boinc_finish(retval);
     }
 
-    do_unzip_inputs();
-
     retval = read_checkpoint(ntasks_completed, checkpoint_cpu_time, runtime);
-    if (retval && !zip_filename.empty()) {
-        // this is the first time we've run.
+    if (retval) {
+        // this is the first time we've run; unzip inputs
+        do_unzip_inputs();
+        write_checkpoint(0, 0, 0);
+
         // If we're going to zip output files,
         // make a list of files present at this point so we can exclude them.
         //
-        write_checkpoint(0, 0, 0);
-        get_initial_file_list();
+        if (!zip_filename.empty()) {
+            get_initial_file_list();
+        }
     }
 
     // do initialization after getting initial file list,
@@ -1290,7 +1323,7 @@ int main(int argc, char** argv) {
         }
     }
 
-    retval = start_daemons(argc, argv);
+    retval = start_daemons(child_args);
     if (retval) {
         fprintf(stderr,
             "%s start_daemons(): %d\n",
@@ -1313,7 +1346,8 @@ int main(int argc, char** argv) {
         double cpu_time = 0;
 
         task.starting_cpu = checkpoint_cpu_time;
-        retval = task.run(argc, argv);
+        retval = task.run(child_args);
+
         if (retval) {
             boinc_finish(retval);
         }
