@@ -1,6 +1,6 @@
 // This file is part of BOINC.
-// http://boinc.berkeley.edu
-// Copyright (C) 2023 University of California
+// https://boinc.berkeley.edu
+// Copyright (C) 2024 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -23,7 +23,7 @@
 #include <regex.h>
 
 // Represents a plan class, as specified in XML
-// if you add anything here, initialize if in the constructor
+// if you add anything here, initialize it in the constructor
 //
 struct PLAN_CLASS_SPEC {
     char name[256];
@@ -32,6 +32,8 @@ struct PLAN_CLASS_SPEC {
     bool cal;
     bool opencl;
     bool virtualbox;
+    bool wsl;
+    bool docker;
     bool is64bit;
     std::vector<std::string> cpu_features;
     double min_ncpus;
@@ -51,6 +53,8 @@ struct PLAN_CLASS_SPEC {
     double max_os_version;
     int min_android_version;
     int max_android_version;
+    int min_libc_version;
+        // if WSL: applies to WSL distro
     char project_prefs_tag[256];
     bool have_project_prefs_regex;
     regex_t project_prefs_regex;
@@ -61,6 +65,7 @@ struct PLAN_CLASS_SPEC {
         // for non-compute-intensive, or override for GPU apps
     bool have_host_summary_regex;
     regex_t host_summary_regex;
+        // matched against host.serialnum
     int user_id;
     double infeasible_random;
     long min_wu_id;
@@ -110,6 +115,9 @@ struct PLAN_CLASS_SPEC {
     int max_opencl_driver_revision;
     bool double_precision_fp;
 
+    // Apple GPU
+    int min_metal_support;
+
     // VirtualBox apps
     //
     int min_vbox_version;
