@@ -566,14 +566,67 @@ static int insert_wu_tags(WORKUNIT& wu, APP& app) {
             wu.keywords
         );
         strcat(buf, buf2);
+        if (config.debug_keyword) {
+            log_messages.printf(MSG_NORMAL,
+                "[keyword] keywords: %s\n", wu.keywords
+            );
+        }
     }
     return insert_after(wu.xml_doc, "<workunit>\n", buf);
 }
 
+// add host usage into to WU's xml_doc (for BUDA jobs)
+//
+static int add_usage_to_wu(WORKUNIT &wu, HOST_USAGE &hu) {
+    char buf[2048], buf2[2048];
+    snprintf(buf, sizeof(buf),
+        "   <avg_ncpus>%f</avg_ncpus>\n"
+        "   <flops>%f</flops>\n",
+        hu.avg_ncpus,
+        hu.projected_flops
+    );
+    if (hu.proc_type != PROC_TYPE_CPU) {
+        snprintf(buf2, sizeof(buf2),
+            "   <coproc>\n"
+            "        <type>%s</type>\n"
+            "        <count>%f</count>\n"
+            "    </coproc>\n",
+            proc_type_name_xml(hu.proc_type),
+            hu.gpu_usage
+        );
+        strcat(buf, buf2);
+    }
+    if (strlen(hu.cmdline)) {
+        snprintf(buf2, sizeof(buf2),
+            "   <cmdline>%s</cmdline>\n",
+            hu.cmdline
+        );
+        strcat(buf, buf2);
+    }
+
+    char *p = wu.xml_doc;
+    if (strlen(p) + strlen(buf) + 10 > sizeof(wu.xml_doc)) {
+        log_messages.printf(MSG_CRITICAL,
+            "add_usage_to_wu(): field too small: %ld %ld %ld\n",
+            strlen(p), strlen(buf), sizeof(wu.xml_doc)
+        );
+        return -1;
+    }
+    p = strstr(p, "</workunit>");
+    if (!p) {
+        log_messages.printf(MSG_CRITICAL, "add_usage_to_wu(): no end tag\n");
+        return -1;
+    }
+    strcpy(p, buf);
+    strcat(p, "</workunit>");
+    return 0;
+}
+
 // Add the given workunit, app, and app version to a reply.
 //
 static int add_wu_to_reply(
-    WORKUNIT& wu, SCHEDULER_REPLY&, APP* app, BEST_APP_VERSION* bavp
+    WORKUNIT& wu, SCHEDULER_REPLY&, APP* app, BEST_APP_VERSION* bavp,
+    bool is_buda, HOST_USAGE &hu
 ) {
     int retval;
     WORKUNIT wu2, wu3;
@@ -626,6 +679,11 @@ static int add_wu_to_reply(
         );
         return retval;
     }
+
+    if (is_buda) {
+        retval = add_usage_to_wu(wu2, hu);
+        if (retval) return retval;
+    }
     wu3 = wu2;
     if (strlen(config.replace_download_url_by_timezone)) {
         process_wu_timezone(wu2, wu3);
@@ -885,10 +943,61 @@ inline static DB_ID_TYPE get_app_version_id(BEST_APP_VERSION* bavp) {
     }
 }
 
+static bool wu_has_plan_class(WORKUNIT &wu, char* buf) {
+    char *p = strstr(wu.xml_doc, "<plan_class>");
+    if (!p) return false;
+    p += strlen("<plan_class>");
+    strncpy(buf, p, 256);
+    p = strstr(buf, "</plan_class>");
+    if (!p) return false;
+    *p = 0;
+    return true;
+}
+
+// If workunit has a plan class (e.g. BUDA)
+//      return false if host not capable
+//      plan class computes host usage
+//      is_buda = true
+// else
+//      host usage is from app version
+//      is_buda = false
+//
+void check_buda_plan_class(
+    WORKUNIT &wu, HOST_USAGE &hu, bool &is_buda, bool &is_ok
+) {
+    char plan_class[256];
+    if (!wu_has_plan_class(wu, plan_class)) {
+        is_buda = false;
+        return;
+    }
+    if (config.debug_version_select) {
+        log_messages.printf(MSG_NORMAL,
+            "[version] BUDA WU plan class: %s\n", plan_class
+        );
+    }
+    is_buda = true;
+    is_ok = true;
+    if (!strlen(plan_class)) {
+        hu.sequential_app(g_reply->host.p_fpops);
+        return;
+    }
+    if (!app_plan(*g_request, plan_class, hu, &wu)) {
+        if (config.debug_version_select) {
+            log_messages.printf(MSG_NORMAL,
+                "[version]  app_plan(%s) returned false\n", plan_class
+            );
+        }
+        // can't send this job
+        is_ok = false;
+    }
+}
+
 int add_result_to_reply(
     SCHED_DB_RESULT& result,
     WORKUNIT& wu,
     BEST_APP_VERSION* bavp,
+    HOST_USAGE &host_usage,
+    bool is_buda,
     bool locality_scheduling
 ) {
     int retval;
@@ -899,7 +1008,7 @@ int add_result_to_reply(
     result.userid = g_reply->user.id;
     result.sent_time = time(0);
     result.report_deadline = result.sent_time + wu.delay_bound;
-    result.flops_estimate = bavp->host_usage.peak_flops;
+    result.flops_estimate = host_usage.peak_flops;
     result.app_version_id = get_app_version_id(bavp);
 
     // update WU DB record.
@@ -964,7 +1073,7 @@ int add_result_to_reply(
     // done with DB updates.
     //
 
-    retval = add_wu_to_reply(wu, *g_reply, app, bavp);
+    retval = add_wu_to_reply(wu, *g_reply, app, bavp, is_buda, host_usage);
     if (retval) return retval;
 
     // Adjust available disk space.
@@ -978,7 +1087,7 @@ int add_result_to_reply(
 
     double est_dur = estimate_duration(wu, *bavp);
     if (config.debug_send) {
-        double max_time = wu.rsc_fpops_bound / bavp->host_usage.projected_flops;
+        double max_time = wu.rsc_fpops_bound / host_usage.projected_flops;
         char buf1[64],buf2[64];
         secs_to_hmsf(est_dur, buf1);
         secs_to_hmsf(max_time, buf2);
@@ -1017,11 +1126,11 @@ int add_result_to_reply(
     // because the scheduling of GPU jobs is constrained by the # of GPUs
     //
     if (g_wreq->rsc_spec_request) {
-        int pt = bavp->host_usage.proc_type;
+        int pt = host_usage.proc_type;
         if (pt == PROC_TYPE_CPU) {
-            double est_cpu_secs = est_dur*bavp->host_usage.avg_ncpus;
+            double est_cpu_secs = est_dur*host_usage.avg_ncpus;
             g_wreq->req_secs[PROC_TYPE_CPU] -= est_cpu_secs;
-            g_wreq->req_instances[PROC_TYPE_CPU] -= bavp->host_usage.avg_ncpus;
+            g_wreq->req_instances[PROC_TYPE_CPU] -= host_usage.avg_ncpus;
             if (config.debug_send_job) {
                 log_messages.printf(MSG_NORMAL,
                     "[send_job] est_dur %f est_cpu_secs %f; new req_secs %f\n",
@@ -1029,9 +1138,9 @@ int add_result_to_reply(
                 );
             }
         } else {
-            double est_gpu_secs = est_dur*bavp->host_usage.gpu_usage;
+            double est_gpu_secs = est_dur*host_usage.gpu_usage;
             g_wreq->req_secs[pt] -= est_gpu_secs;
-            g_wreq->req_instances[pt] -= bavp->host_usage.gpu_usage;
+            g_wreq->req_instances[pt] -= host_usage.gpu_usage;
             if (config.debug_send_job) {
                 log_messages.printf(MSG_NORMAL,
                     "[send_job] est_dur %f est_gpu_secs %f; new req_secs %f\n",
@@ -1045,7 +1154,7 @@ int add_result_to_reply(
     }
     update_estimated_delay(*bavp, est_dur);
     g_wreq->njobs_sent++;
-    config.max_jobs_in_progress.register_job(app, bavp->host_usage.proc_type);
+    config.max_jobs_in_progress.register_job(app, host_usage.proc_type);
     if (!resent_result) {
         DB_HOST_APP_VERSION* havp = bavp->host_app_version();
         if (havp) {
@@ -1230,7 +1339,13 @@ void send_gpu_messages() {
             proc_type_name(PROC_TYPE_INTEL_GPU)
         );
     }
-
+    if (g_request->coprocs.apple_gpu.count && ssp->have_apps_for_proc_type[PROC_TYPE_APPLE_GPU]) {
+        send_gpu_property_messages(gpu_requirements[PROC_TYPE_APPLE_GPU],
+            g_request->coprocs.apple_gpu.opencl_prop.global_mem_size,
+            0,
+            proc_type_name(PROC_TYPE_APPLE_GPU)
+        );
+    }
 }
 
 // send messages to user about why jobs were or weren't sent,
