@@ -28,6 +28,7 @@
 #include <sys/param.h>
 
 using std::vector;
+using std::string;
 
 #include "boinc_db.h"
 #include "error_numbers.h"
@@ -107,6 +108,20 @@ static void overflow(const char* table, const char* param_name) {
     exit(1);
 }
 
+void get_buda_plan_classes(vector<string> &pcs) {
+    pcs.clear();
+    FILE *f = boinc::fopen("../buda_plan_classes", "r");
+    if (!f) return;
+    char buf[256];
+    while (boinc::fgets(buf, 256, f)) {
+        strip_whitespace(buf);
+        pcs.push_back(buf);
+    }
+    boinc::fclose(f);
+}
+
+// scan various DB tables and populate shared-memory arrays
+//
 int SCHED_SHMEM::scan_tables() {
     DB_PLATFORM platform;
     DB_APP app;
@@ -243,6 +258,12 @@ int SCHED_SHMEM::scan_tables() {
         int rt = plan_class_to_proc_type(av.plan_class);
         have_apps_for_proc_type[rt] = true;
     }
+    vector<string> buda_plan_classes;
+    get_buda_plan_classes(buda_plan_classes);
+    for (string pc: buda_plan_classes) {
+        int rt = plan_class_to_proc_type(pc.c_str());
+        have_apps_for_proc_type[rt] = true;
+    }
     for (i=0; i<NPROC_TYPES; i++) {
         boinc::fprintf(stderr, "have apps for %s: %s\n",
             proc_type_name(i),
@@ -392,7 +413,7 @@ void SCHED_SHMEM::show(FILE* f) {
                 "WU ID",
                 "result ID",
                 "batch",
-                "HR class",
+                "kwds",
                 "priority",
                 "in shmem",
                 "size class",
@@ -411,13 +432,13 @@ void SCHED_SHMEM::show(FILE* f) {
             appname = app?app->name:"missing";
             delta_t = dtime() - wu_result.time_added_to_shared_memory;
             boinc::fprintf(f,
-                "%4d %12.12s %10lu %10lu %10d %8d %10d %7ds %9d %12s %9d %9d\n",
+                "%4d %12.12s %10lu %10lu %10d %8s %10d %7ds %9d %12s %9d %9d\n",
                 i,
                 appname,
                 wu_result.workunit.id,
                 wu_result.resultid,
                 wu_result.workunit.batch,
-                wu_result.workunit.hr_class,
+                wu_result.workunit.keywords,
                 wu_result.res_priority,
                 delta_t,
                 wu_result.workunit.size_class,
